@@ -2,9 +2,9 @@
 #
 # A Python Gtk UI for Hatari that can embed the Hatari emulator window.
 #
-# Requires Gtk 3.x and Python GObject Introspection libraries.
+# Requires Gtk 3.x and Python GLib Introspection libraries.
 #
-# Copyright (C) 2008-2019 by Eero Tamminen
+# Copyright (C) 2008-2020 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ import gi
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GObject
+from gi.repository import GLib
 
 from debugui import HatariDebugUI
 from hatari import Hatari, HatariConfigMapping
@@ -55,16 +55,24 @@ class UICallbacks:
         error = self.hatari.is_compatible()
         if error:
             ErrorDialog(None).run(error)
-            sys.exit(-1)
+            sys.exit(1)
 
         self.config = HatariConfigMapping(self.hatari)
         try:
             self.config.validate()
         except (KeyError, AttributeError):
-            NoteDialog(None).run("Loading Hatari configuration failed!\nRetrying after saving Hatari configuration.")
-            self.hatari.save_config()
+            NoteDialog(None).run("Hatari configuration validation failed!\nRetrying after saving Hatari configuration.")
+            error = self.hatari.save_config()
+            if error:
+                ErrorDialog(None).run("Hatari configuration saving failed (code: %d), quitting!" % error)
+                sys.exit(error)
             self.config = HatariConfigMapping(self.hatari)
-            self.config.validate()
+            try:
+                self.config.validate()
+            except (KeyError, AttributeError):
+                ErrorDialog(None).run("Invalid Hatari configuration, quitting!")
+                sys.exit(1)
+        self.config.init_compat()
 
         # windows are created when needed
         self.mainwin = None
@@ -121,7 +129,7 @@ class UICallbacks:
         # add vertical elements
         vbox = Gtk.VBox()
         if menu:
-            vbox.add(menu)
+            vbox.pack_start(menu, False, True, 0)
         if toolbars["top"]:
             vbox.pack_start(toolbars["top"], False, True, 0)
         vbox.add(hbox)
@@ -150,17 +158,20 @@ class UICallbacks:
         socket.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("black"))
         socket.set_events(Gdk.EventMask.ALL_EVENTS_MASK)
         # set max Hatari window size = desktop size
-        self.config.set_desktop_size(Gdk.Screen.width(), Gdk.Screen.height())
+        monitor  = Gdk.Display.get_default().get_primary_monitor()
+        geometry = monitor.get_geometry()
+        scale    = monitor.get_scale_factor()
+        self.config.set_desktop_size(scale * geometry.width, scale * geometry.height)
         # set initial embedded hatari size
         width, height = self.config.get_window_size()
         socket.set_size_request(width, height)
-        # no resizing for the Hatari window
-        box.pack_start(socket, False, False, 0)
+        # allow Hatari window resizing
+        box.pack_start(socket, True, True, 0)
         self.hatariwin = socket
 
     # ------- run callback -----------
     def _socket_cb(self, fd, event):
-        if event != GObject.IO_IN:
+        if event != GLib.IO_IN:
             # hatari process died, make sure Hatari instance notices
             self.hatari.kill()
             return False
@@ -177,7 +188,7 @@ class UICallbacks:
         if not self.killdialog.run(self.hatari):
             return
         if self.io_id:
-            GObject.source_remove(self.io_id)
+            GLib.source_remove(self.io_id)
         args = ["--configfile"]
         # whether to use Hatari config or unsaved Hatari UI config?
         if self.config.is_changed():
@@ -194,8 +205,8 @@ class UICallbacks:
             # get notifications of Hatari window size changes
             self.hatari.enable_embed_info()
             socket = self.hatari.get_control_socket().fileno()
-            events = GObject.IO_IN | GObject.IO_HUP | GObject.IO_ERR
-            self.io_id = GObject.io_add_watch(socket, events, self._socket_cb)
+            events = GLib.IO_IN | GLib.IO_HUP | GLib.IO_ERR
+            self.io_id = GLib.io_add_watch(socket, events, self._socket_cb)
             # all keyboard events should go to Hatari window
             self.hatariwin.grab_focus()
         else:
@@ -207,15 +218,15 @@ class UICallbacks:
     # ------- quit callback -----------
     def quit(self, widget, arg = None):
         # due to Gtk API, needs to return True when *not* quitting
-        if not self.killdialog.run(self.hatari):
-            return True
-        if self.io_id:
-            GObject.source_remove(self.io_id)
         if self.config.is_changed():
             if not self.quitdialog:
                 self.quitdialog = QuitSaveDialog(self.mainwin)
             if not self.quitdialog.run(self.config):
                 return True
+        if not self.killdialog.run(self.hatari):
+            return True
+        if self.io_id:
+            GLib.source_remove(self.io_id)
         Gtk.main_quit()
         if os.path.exists(self.tmpconfpath):
             os.unlink(self.tmpconfpath)
@@ -661,7 +672,7 @@ class UIActions:
 
         # ugly, Hatari socket window ID can be gotten only
         # after Socket window is realized by gtk_main()
-        GObject.idle_add(self.callbacks.run)
+        GLib.idle_add(self.callbacks.run)
         Gtk.main()
 
 
