@@ -9,7 +9,7 @@
   The configuration file is now stored in an ASCII format to allow the user
   to edit the file manually.
 */
-const char Configuration_fileid[] = "Hatari configuration.c : " __DATE__ " " __TIME__;
+const char Configuration_fileid[] = "Hatari configuration.c";
 
 #include <SDL_keyboard.h>
 #include <SDL_joystick.h>
@@ -62,10 +62,11 @@ static const struct Config_Tag configs_Debugger[] =
 	{ "nSymbolLines", Int_Tag, &ConfigureParams.Debugger.nSymbolLines },
 	{ "nMemdumpLines", Int_Tag, &ConfigureParams.Debugger.nMemdumpLines },
 	{ "nDisasmLines", Int_Tag, &ConfigureParams.Debugger.nDisasmLines },
+	{ "nBacktraceLines", Int_Tag, &ConfigureParams.Debugger.nBacktraceLines },
 	{ "nExceptionDebugMask", Int_Tag, &ConfigureParams.Debugger.nExceptionDebugMask },
 	{ "nDisasmOptions", Int_Tag, &ConfigureParams.Debugger.nDisasmOptions },
 	{ "bDisasmUAE", Bool_Tag, &ConfigureParams.Debugger.bDisasmUAE },
-	{ "bSymbolsResident", Bool_Tag, &ConfigureParams.Debugger.bSymbolsResident },
+	{ "bSymbolsAutoLoad", Bool_Tag, &ConfigureParams.Debugger.bSymbolsAutoLoad },
 	{ "bMatchAllSymbols", Bool_Tag, &ConfigureParams.Debugger.bMatchAllSymbols },
 	{ NULL , Error_Tag, NULL }
 };
@@ -98,8 +99,8 @@ static const struct Config_Tag configs_Screen[] =
 	{ "nMaxWidth", Int_Tag, &ConfigureParams.Screen.nMaxWidth },
 	{ "nMaxHeight", Int_Tag, &ConfigureParams.Screen.nMaxHeight },
 #if WITH_SDL2
+	{ "nZoomFactor", Float_Tag, &ConfigureParams.Screen.nZoomFactor },
 	{ "bUseSdlRenderer", Bool_Tag, &ConfigureParams.Screen.bUseSdlRenderer },
-	{ "nRenderScaleQuality", Int_Tag, &ConfigureParams.Screen.nRenderScaleQuality },
 	{ "bUseVsync", Bool_Tag, &ConfigureParams.Screen.bUseVsync },
 #endif
 	{ NULL , Error_Tag, NULL }
@@ -540,6 +541,19 @@ static const struct Config_Tag configs_Rom[] =
 	{ NULL , Error_Tag, NULL }
 };
 
+/* Used to load/save LILO options, names are same as with Aranym */
+static const struct Config_Tag configs_Lilo[] =
+{
+	{ "Args", String_Tag, ConfigureParams.Lilo.szCommandLine },
+	{ "Kernel", String_Tag, ConfigureParams.Lilo.szKernelFileName },
+	{ "Symbols", String_Tag, ConfigureParams.Lilo.szKernelSymbols },
+	{ "Ramdisk", String_Tag, ConfigureParams.Lilo.szRamdiskFileName },
+	{ "HaltOnReboot", Bool_Tag, &ConfigureParams.Lilo.bHaltOnReboot },
+	{ "KernelToFastRam", Bool_Tag, &ConfigureParams.Lilo.bKernelToFastRam },
+	{ "RamdiskToFastRam", Bool_Tag, &ConfigureParams.Lilo.bRamdiskToFastRam },
+	{ NULL , Error_Tag, NULL }
+};
+
 /* Used to load/save RS232 options */
 static const struct Config_Tag configs_Rs232[] =
 {
@@ -645,10 +659,11 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Debugger.nSymbolLines = -1; /* <0: use terminal size */
 	ConfigureParams.Debugger.nMemdumpLines = -1; /* <0: use terminal size */
 	ConfigureParams.Debugger.nDisasmLines = -1; /* <0: use terminal size */
+	ConfigureParams.Debugger.nBacktraceLines = 0; /* <=0: show all */
 	ConfigureParams.Debugger.nExceptionDebugMask = DEFAULT_EXCEPTIONS;
 	/* external one has nicer output, but isn't as complete as UAE one */
-	ConfigureParams.Debugger.bDisasmUAE = false;
-	ConfigureParams.Debugger.bSymbolsResident = false;
+	ConfigureParams.Debugger.bDisasmUAE = true;
+	ConfigureParams.Debugger.bSymbolsAutoLoad = true;
 	ConfigureParams.Debugger.bMatchAllSymbols = false;
 	ConfigureParams.Debugger.nDisasmOptions = Disasm_GetOptions();
 
@@ -771,15 +786,18 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Memory.STRamSize_KB = 1024;	/* 1 MiB */
 	ConfigureParams.Memory.TTRamSize_KB = 0;	/* disabled */
 	ConfigureParams.Memory.bAutoSave = false;
-	sprintf(ConfigureParams.Memory.szMemoryCaptureFileName, "%s%chatari.sav",
-	        psHomeDir, PATHSEP);
-	sprintf(ConfigureParams.Memory.szAutoSaveFileName, "%s%cauto.sav",
-	        psHomeDir, PATHSEP);
+	File_MakePathBuf(ConfigureParams.Memory.szMemoryCaptureFileName,
+	                 sizeof(ConfigureParams.Memory.szMemoryCaptureFileName),
+	                 psHomeDir, "hatari", "sav");
+	File_MakePathBuf(ConfigureParams.Memory.szAutoSaveFileName,
+	                 sizeof(ConfigureParams.Memory.szAutoSaveFileName),
+	                 psHomeDir, "auto", "sav");
 
 	/* Set defaults for Printer */
 	ConfigureParams.Printer.bEnablePrinting = false;
-	sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%chatari.prn",
-	        psHomeDir, PATHSEP);
+	File_MakePathBuf(ConfigureParams.Printer.szPrintToFileName,
+	                 sizeof(ConfigureParams.Printer.szPrintToFileName),
+	                 psHomeDir, "hatari", "prn");
 
 	/* Set defaults for MFP RS232 (ST/MegaST/STE/MegaSTE/TT) */
 	ConfigureParams.RS232.bEnableRS232 = false;
@@ -819,14 +837,17 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Screen.bShowStatusbar = true;
 	ConfigureParams.Screen.bShowDriveLed = true;
 	ConfigureParams.Screen.bCrop = false;
-	/* gives zoomed Falcon/TT windows about same size as ST/STE windows */
+	/* use approximately similar Hatari framebuffer/window size
+	 * on all resolutions (like real Atari monitor would do) by
+	 * allowing ST low resolution to be doubled (see screen.c)
+	 */
 	ConfigureParams.Screen.nMaxWidth = 2*NUM_VISIBLE_LINE_PIXELS;
 	ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+STATUSBAR_MAX_HEIGHT;
 	ConfigureParams.Screen.bForceMax = false;
 	ConfigureParams.Screen.DisableVideo = false;
 #if WITH_SDL2
+	ConfigureParams.Screen.nZoomFactor = 1.0;
 	ConfigureParams.Screen.bUseSdlRenderer = true;
-	ConfigureParams.Screen.nRenderScaleQuality = 0;
 	ConfigureParams.Screen.bUseVsync = false;
 #endif
 
@@ -835,17 +856,33 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Sound.bEnableSound = true;
 	ConfigureParams.Sound.bEnableSoundSync = false;
 	ConfigureParams.Sound.nPlaybackFreq = 44100;
-	sprintf(ConfigureParams.Sound.szYMCaptureFileName, "%s%chatari.wav",
-	        psWorkingDir, PATHSEP);
+	File_MakePathBuf(ConfigureParams.Sound.szYMCaptureFileName,
+	                 sizeof(ConfigureParams.Sound.szYMCaptureFileName),
+	                 psWorkingDir, "hatari", "wav");
 	ConfigureParams.Sound.SdlAudioBufferSize = 0;
 	ConfigureParams.Sound.YmVolumeMixing = YM_TABLE_MIXING;
 
 	/* Set defaults for Rom */
-	sprintf(ConfigureParams.Rom.szTosImageFileName, "%s%ctos.img",
-	        Paths_GetDataDir(), PATHSEP);
+	File_MakePathBuf(ConfigureParams.Rom.szTosImageFileName,
+	                 sizeof(ConfigureParams.Rom.szTosImageFileName),
+	                 Paths_GetDataDir(), "tos", "img");
 	ConfigureParams.Rom.bPatchTos = true;
 	strcpy(ConfigureParams.Rom.szCartridgeImageFileName, "");
 
+	/* Set defaults for Lilo */
+	strcpy(ConfigureParams.Lilo.szCommandLine,
+	       "root=/dev/ram video=atafb:vga16 load_ramdisk=1");
+	File_MakePathBuf(ConfigureParams.Lilo.szKernelFileName,
+	                 sizeof(ConfigureParams.Lilo.szKernelFileName),
+	                 Paths_GetDataDir(), "vmlinuz", NULL);
+	File_MakePathBuf(ConfigureParams.Lilo.szRamdiskFileName,
+	                 sizeof(ConfigureParams.Lilo.szRamdiskFileName),
+	                 Paths_GetDataDir(), "initrd", NULL);
+	ConfigureParams.Lilo.szKernelSymbols[0] = '\0';
+	ConfigureParams.Lilo.bRamdiskToFastRam = true;
+	ConfigureParams.Lilo.bKernelToFastRam = true;
+	ConfigureParams.Lilo.bHaltOnReboot = true;
+
 	/* Set defaults for System */
 	ConfigureParams.System.nMachineType = MACHINE_ST;
 	ConfigureParams.System.nCpuLevel = 0;
@@ -873,18 +910,16 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Video.AviRecordVcodec = AVI_RECORD_VIDEO_CODEC_BMP;
 #endif
 	ConfigureParams.Video.AviRecordFps = 0;			/* automatic FPS */
-	sprintf(ConfigureParams.Video.AviRecordFile, "%s%chatari.avi", psWorkingDir, PATHSEP);
+	File_MakePathBuf(ConfigureParams.Video.AviRecordFile,
+	                 sizeof(ConfigureParams.Video.AviRecordFile),
+	                 psWorkingDir, "hatari", "avi");
 
 	/* Initialize the configuration file name */
-	if (strlen(psHomeDir) < sizeof(sConfigFileName)-13)
-		sprintf(sConfigFileName, "%s%chatari.cfg", psHomeDir, PATHSEP);
-	else
+	if (File_MakePathBuf(sConfigFileName, sizeof(sConfigFileName),
+	                     psHomeDir, "hatari", "cfg"))
+	{
 		strcpy(sConfigFileName, "hatari.cfg");
-
-#if defined(__AMIGAOS4__)
-	/* Fix default path names on Amiga OS */
-	sprintf(ConfigureParams.Rom.szTosImageFileName, "%stos.img", Paths_GetDataDir());
-#endif
+	}
 }
 
 
@@ -926,7 +961,11 @@ void Configuration_Apply(bool bReset)
 	/* Check/convert ST RAM size in KB */
 	size = STMemory_RAM_Validate_Size_KB ( ConfigureParams.Memory.STRamSize_KB );
 	if ( size < 0 )
+	{
 		size = 1024;
+		Log_Printf(LOG_WARN, "Unsupported %d KB ST-RAM amount, defaulting to %d KB\n",
+			   ConfigureParams.Memory.STRamSize_KB, size);
+	}
 	ConfigureParams.Memory.STRamSize_KB = size;
 	STMemory_Init ( ConfigureParams.Memory.STRamSize_KB * 1024 );
 
@@ -976,6 +1015,12 @@ void Configuration_Apply(bool bReset)
 	File_MakeAbsoluteName(ConfigureParams.Rom.szTosImageFileName);
 	if (strlen(ConfigureParams.Rom.szCartridgeImageFileName) > 0)
 		File_MakeAbsoluteName(ConfigureParams.Rom.szCartridgeImageFileName);
+	if (strlen(ConfigureParams.Lilo.szKernelFileName) > 0)
+		File_MakeAbsoluteName(ConfigureParams.Lilo.szKernelFileName);
+	if (strlen(ConfigureParams.Lilo.szKernelSymbols) > 0)
+		File_MakeAbsoluteName(ConfigureParams.Lilo.szKernelSymbols);
+	if (strlen(ConfigureParams.Lilo.szRamdiskFileName) > 0)
+		File_MakeAbsoluteName(ConfigureParams.Lilo.szRamdiskFileName);
 	File_CleanFileName(ConfigureParams.HardDisk.szHardDiskDirectories[0]);
 	File_MakeAbsoluteName(ConfigureParams.HardDisk.szHardDiskDirectories[0]);
 	File_MakeAbsoluteName(ConfigureParams.Memory.szMemoryCaptureFileName);
@@ -1123,6 +1168,7 @@ void Configuration_Load(const char *psFileName)
 	Configuration_LoadSection(psFileName, configs_Scsi, "[SCSI]");
 	Configuration_LoadSection(psFileName, configs_Ide, "[IDE]");
 	Configuration_LoadSection(psFileName, configs_Rom, "[ROM]");
+	Configuration_LoadSection(psFileName, configs_Lilo, "[LILO]");
 	Configuration_LoadSection(psFileName, configs_Rs232, "[RS232]");
 	Configuration_LoadSection(psFileName, configs_Printer, "[Printer]");
 	Configuration_LoadSection(psFileName, configs_Midi, "[Midi]");
@@ -1179,6 +1225,7 @@ void Configuration_Save(void)
 	Configuration_SaveSection(sConfigFileName, configs_Scsi, "[SCSI]");
 	Configuration_SaveSection(sConfigFileName, configs_Ide, "[IDE]");
 	Configuration_SaveSection(sConfigFileName, configs_Rom, "[ROM]");
+	Configuration_SaveSection(sConfigFileName, configs_Lilo, "[LILO]");
 	Configuration_SaveSection(sConfigFileName, configs_Rs232, "[RS232]");
 	Configuration_SaveSection(sConfigFileName, configs_Printer, "[Printer]");
 	Configuration_SaveSection(sConfigFileName, configs_Midi, "[Midi]");
@@ -1199,6 +1246,9 @@ void Configuration_MemorySnapShot_Capture(bool bSave)
 	MemorySnapShot_Store(ConfigureParams.Rom.szTosImageFileName, sizeof(ConfigureParams.Rom.szTosImageFileName));
 	MemorySnapShot_Store(ConfigureParams.Rom.szCartridgeImageFileName, sizeof(ConfigureParams.Rom.szCartridgeImageFileName));
 
+	MemorySnapShot_Store(ConfigureParams.Lilo.szKernelFileName, sizeof(ConfigureParams.Lilo.szKernelFileName));
+	MemorySnapShot_Store(ConfigureParams.Lilo.szRamdiskFileName, sizeof(ConfigureParams.Lilo.szRamdiskFileName));
+
 	MemorySnapShot_Store(&ConfigureParams.Memory.STRamSize_KB, sizeof(ConfigureParams.Memory.STRamSize_KB));
 	MemorySnapShot_Store(&ConfigureParams.Memory.TTRamSize_KB, sizeof(ConfigureParams.Memory.TTRamSize_KB));
 
