@@ -7,6 +7,11 @@
 * Andreas Grabher and Toni Wilen
 *
 */
+
+#ifdef WIN32 /* WINUAE_FOR_HATARI */
+#define __USE_MINGW_ANSI_STDIO 1        /* Hack for %lld format specifiers */
+#endif
+
 #define __USE_ISOC9X  /* We might be able to pick up a NaN */
 
 #define SOFTFLOAT_FAST_INT64
@@ -132,7 +137,7 @@ static const TCHAR *fp_printx80(floatx80 *fx, int mode)
 		int8_t save_exception_flags = fs.float_exception_flags;
 		fs.float_exception_flags = 0;
 		floatx80 x = floatx80_to_floatdecimal(*fx, &len, &fs);
-		_stprintf(fsout, _T("%c%01lld.%016llde%c%04d%s%s"), n ? '-' : '+',
+		_stprintf(fsout, _T("%c%01lld.%016llde%c%05u%s%s"), n ? '-' : '+',
 				x.low / LIT64(10000000000000000), x.low % LIT64(10000000000000000),
 				(x.high & 0x4000) ? '-' : '+', x.high & 0x3FFF, d ? _T("D") : u ? _T("U") : _T(""),
 				(fs.float_exception_flags & float_flag_inexact) ? _T("~") : _T(""));
