@@ -9,7 +9,7 @@
 #ifndef UAE_MEMORY_H
 #define UAE_MEMORY_H
 
-extern void memory_reset (void);
+extern void memory_reset(void);
 
 #ifdef JIT
 extern int special_mem;
@@ -82,7 +82,7 @@ extern uae_u8* baseaddr[];
 #define CACHE_DISABLE_MMU 0x10
 extern uae_u8 ce_banktype[65536], ce_cachable[65536];
 
-#define ABFLAG_CACHE_SHIFT 16
+#define ABFLAG_CACHE_SHIFT 24
 enum
 {
 	ABFLAG_UNK = 0, ABFLAG_RAM = 1, ABFLAG_ROM = 2, ABFLAG_ROMIN = 4, ABFLAG_IO = 8,
@@ -90,6 +90,7 @@ enum
 	ABFLAG_RTG = 256, ABFLAG_THREADSAFE = 512, ABFLAG_DIRECTMAP = 1024, ABFLAG_ALLOCINDIRECT = 2048,
 	ABFLAG_CHIPRAM = 4096, ABFLAG_CIA = 8192, ABFLAG_PPCIOSPACE = 16384,
 	ABFLAG_MAPPED = 32768,
+	ABFLAG_DIRECTACCESS = 65536,
 	ABFLAG_CACHE_ENABLE_DATA = CACHE_ENABLE_DATA << ABFLAG_CACHE_SHIFT,
 	ABFLAG_CACHE_ENABLE_DATA_BURST = CACHE_ENABLE_DATA_BURST << ABFLAG_CACHE_SHIFT,
 	ABFLAG_CACHE_ENABLE_INS = CACHE_ENABLE_INS << ABFLAG_CACHE_SHIFT,
@@ -132,6 +133,10 @@ typedef struct {
 	uae_u32 allocated_size;
 	// size of bank (if IO or before RAM allocation)
 	uae_u32 reserved_size;
+	/* non-NULL if xget/xput can be bypassed */
+	uae_u8 *baseaddr_direct_r;
+	uae_u8 *baseaddr_direct_w;
+	uae_u32 startaccessmask;
 } addrbank;
 
 #define MEMORY_MIN_SUBBANK 1024
@@ -196,7 +201,7 @@ static uae_u32 REGPARAM3 name ## _lgeti (uaecptr) REGPARAM; \
 static uae_u32 REGPARAM2 name ## _lgeti (uaecptr addr) \
 { \
 	uae_u8 *m; \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	return do_get_mem_long ((uae_u32 *)m); \
@@ -206,7 +211,7 @@ static uae_u32 REGPARAM3 name ## _wgeti (uaecptr) REGPARAM; \
 static uae_u32 REGPARAM2 name ## _wgeti (uaecptr addr) \
 { \
 	uae_u8 *m; \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	return do_get_mem_word ((uae_u16 *)m); \
@@ -216,7 +221,7 @@ static uae_u32 REGPARAM3 name ## _lget (uaecptr) REGPARAM; \
 static uae_u32 REGPARAM2 name ## _lget (uaecptr addr) \
 { \
 	uae_u8 *m; \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	return do_get_mem_long ((uae_u32 *)m); \
@@ -226,7 +231,7 @@ static uae_u32 REGPARAM3 name ## _wget (uaecptr) REGPARAM; \
 static uae_u32 REGPARAM2 name ## _wget (uaecptr addr) \
 { \
 	uae_u8 *m; \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	return do_get_mem_word ((uae_u16 *)m); \
@@ -235,7 +240,7 @@ static uae_u32 REGPARAM2 name ## _wget (uaecptr addr) \
 static uae_u32 REGPARAM3 name ## _bget (uaecptr) REGPARAM; \
 static uae_u32 REGPARAM2 name ## _bget (uaecptr addr) \
 { \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	return name ## _bank.baseaddr[addr]; \
 }
@@ -244,7 +249,7 @@ static void REGPARAM3 name ## _lput (uaecptr, uae_u32) REGPARAM; \
 static void REGPARAM2 name ## _lput (uaecptr addr, uae_u32 l) \
 { \
 	uae_u8 *m;  \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	do_put_mem_long ((uae_u32 *)m, l); \
@@ -254,7 +259,7 @@ static void REGPARAM3 name ## _wput (uaecptr, uae_u32) REGPARAM; \
 static void REGPARAM2 name ## _wput (uaecptr addr, uae_u32 w) \
 { \
 	uae_u8 *m;  \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	m = name ## _bank.baseaddr + addr; \
 	do_put_mem_word ((uae_u16 *)m, w); \
@@ -263,7 +268,7 @@ static void REGPARAM2 name ## _wput (uaecptr addr, uae_u32 w) \
 static void REGPARAM3 name ## _bput (uaecptr, uae_u32) REGPARAM; \
 static void REGPARAM2 name ## _bput (uaecptr addr, uae_u32 b) \
 { \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	name ## _bank.baseaddr[addr] = b; \
 }
@@ -271,7 +276,7 @@ static void REGPARAM2 name ## _bput (uaecptr addr, uae_u32 b) \
 static int REGPARAM3 name ## _check (uaecptr addr, uae_u32 size) REGPARAM; \
 static int REGPARAM2 name ## _check (uaecptr addr, uae_u32 size) \
 { \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	return (addr + size) <= name ## _bank.allocated_size; \
 }
@@ -279,7 +284,7 @@ static int REGPARAM2 name ## _check (uaecptr addr, uae_u32 size) \
 static uae_u8 *REGPARAM3 name ## _xlate (uaecptr addr) REGPARAM; \
 static uae_u8 *REGPARAM2 name ## _xlate (uaecptr addr) \
 { \
-	addr -= name ## _bank.start & name ## _bank.mask; \
+	addr -= name ## _bank.startaccessmask; \
 	addr &= name ## _bank.mask; \
 	return name ## _bank.baseaddr + addr; \
 }
@@ -434,14 +439,15 @@ extern addrbank extendedkickmem_bank;
 extern addrbank extendedkickmem2_bank;
 extern addrbank custmem1_bank;
 extern addrbank custmem2_bank;
+extern addrbank romboardmem_bank[MAX_ROM_BOARDS];
 
 extern void rtarea_init(void);
 extern void rtarea_free(void);
 extern void rtarea_init_mem(void);
 extern void rtarea_setup(void);
-extern void expamem_reset (void);
-extern void expamem_next (addrbank *mapped, addrbank *next);
-extern void expamem_shutup (addrbank *mapped);
+extern void expamem_reset(int);
+extern void expamem_next(addrbank *mapped, addrbank *next);
+extern void expamem_shutup(addrbank *mapped);
 extern bool expamem_z3hack(struct uae_prefs*);
 extern void expansion_cpu_fallback(void);
 extern void set_expamem_z3_hack_mode(int);
@@ -504,6 +510,7 @@ extern addrbank *get_mem_bank_real(uaecptr);
 
 #ifdef WINUAE_FOR_HATARI
 extern bool memory_region_bus_error ( uaecptr addr );
+extern bool memory_region_iomem ( uaecptr addr );
 extern void memory_map_Standard_RAM ( Uint32 MMU_Bank0_Size , Uint32 MMU_Bank1_Size );
 #endif
 extern void memory_init(uae_u32 NewSTMemSize, uae_u32 NewTTMemSize, uae_u32 NewRomMemStart);
@@ -531,75 +538,118 @@ extern bool read_kickstart_version(struct uae_prefs *p);
 extern void chipmem_setindirect(void);
 #endif
 
-#define longget(addr) (call_mem_get_func(get_mem_bank(addr).lget, addr))
-#define wordget(addr) (call_mem_get_func(get_mem_bank(addr).wget, addr))
-#define byteget(addr) (call_mem_get_func(get_mem_bank(addr).bget, addr))
-#define longgeti(addr) (call_mem_get_func(get_mem_bank(addr).lgeti, addr))
-#define wordgeti(addr) (call_mem_get_func(get_mem_bank(addr).wgeti, addr))
-#define longput(addr,l) (call_mem_put_func(get_mem_bank(addr).lput, addr, l))
-#define wordput(addr,w) (call_mem_put_func(get_mem_bank(addr).wput, addr, w))
-#define byteput(addr,b) (call_mem_put_func(get_mem_bank(addr).bput, addr, b))
+uae_u32 memory_get_long(uaecptr);
+uae_u32 memory_get_word(uaecptr);
+uae_u32 memory_get_byte(uaecptr);
+uae_u32 memory_get_longi(uaecptr);
+uae_u32 memory_get_wordi(uaecptr);
 
-STATIC_INLINE uae_u32 get_long (uaecptr addr)
+STATIC_INLINE uae_u32 get_long(uaecptr addr)
 {
-	return longget (addr);
+	return memory_get_long(addr);
 }
 STATIC_INLINE uae_u32 get_word (uaecptr addr)
 {
-	return wordget (addr);
+	return memory_get_word(addr);
 }
 STATIC_INLINE uae_u32 get_byte (uaecptr addr)
 {
-	return byteget (addr);
+	return memory_get_byte(addr);
 }
 STATIC_INLINE uae_u32 get_longi(uaecptr addr)
 {
-	return longgeti (addr);
+	return memory_get_longi(addr);
 }
 STATIC_INLINE uae_u32 get_wordi(uaecptr addr)
 {
-	return wordgeti (addr);
+	return memory_get_wordi(addr);
 }
 
+// do split memory access if it can cross memory banks
+STATIC_INLINE uae_u32 get_long_compatible(uaecptr addr)
+{
+	if ((addr &0xffff) < 0xfffd) {
+		return memory_get_long(addr);
+	} else if (addr & 1) {
+		uae_u8 v0 = memory_get_byte(addr + 0);
+		uae_u16 v1 = memory_get_word(addr + 1);
+		uae_u8 v3 = memory_get_byte(addr + 3);
+		return (v0 << 24) | (v1 << 8) | (v3 << 0);
+	} else {
+		uae_u16 v0 = memory_get_word(addr + 0);
+		uae_u16 v1 = memory_get_word(addr + 2);
+		return (v0 << 16) | (v1 << 0);
+	}
+}
+STATIC_INLINE uae_u32 get_word_compatible(uaecptr addr)
+{
+	if ((addr & 0xffff) < 0xffff) {
+		return memory_get_word(addr);
+	} else {
+		uae_u8 v0 = memory_get_byte(addr + 0);
+		uae_u8 v1 = memory_get_byte(addr + 1);
+		return (v0 << 8) | (v1 << 0);
+	}
+}
+STATIC_INLINE uae_u32 get_byte_compatible(uaecptr addr)
+{
+	return memory_get_byte(addr);
+}
+STATIC_INLINE uae_u32 get_longi_compatible(uaecptr addr)
+{
+	if ((addr & 0xffff) < 0xfffd) {
+		return memory_get_longi(addr);
+	} else {
+		uae_u16 v0 = memory_get_wordi(addr + 0);
+		uae_u16 v1 = memory_get_wordi(addr + 2);
+		return (v0 << 16) | (v1 << 0);
+	}
+}
+STATIC_INLINE uae_u32 get_wordi_compatible(uaecptr addr)
+{
+	return memory_get_wordi(addr);
+}
+
+
 STATIC_INLINE uae_u32 get_long_jit(uaecptr addr)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_read_flag;
 #endif
-	return bank->lget(addr);
+	return memory_get_long(addr);
 }
 STATIC_INLINE uae_u32 get_word_jit(uaecptr addr)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_read_flag;
 #endif
-	return bank->wget(addr);
+	return memory_get_word(addr);
 }
 STATIC_INLINE uae_u32 get_byte_jit(uaecptr addr)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_read_flag;
 #endif
-	return bank->bget(addr);
+	return memory_get_byte(addr);
 }
 STATIC_INLINE uae_u32 get_longi_jit(uaecptr addr)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_read_flag;
 #endif
-	return bank->lgeti(addr);
+	return memory_get_longi(addr);
 }
 STATIC_INLINE uae_u32 get_wordi_jit(uaecptr addr)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_read_flag;
 #endif
-	return bank->wgeti(addr);
+	return memory_get_wordi(addr);
 }
 
 /*
@@ -632,42 +682,75 @@ STATIC_INLINE void *get_pointer (uaecptr addr)
 # endif
 #endif
 
+void memory_put_long(uaecptr, uae_u32);
+void memory_put_word(uaecptr, uae_u32);
+void memory_put_byte(uaecptr, uae_u32);
+
 STATIC_INLINE void put_long (uaecptr addr, uae_u32 l)
 {
-	longput(addr, l);
+	memory_put_long(addr, l);
 }
 STATIC_INLINE void put_word (uaecptr addr, uae_u32 w)
 {
-	wordput(addr, w);
+	memory_put_word(addr, w);
 }
 STATIC_INLINE void put_byte (uaecptr addr, uae_u32 b)
 {
-	byteput(addr, b);
+	memory_put_byte(addr, b);
 }
 
+// do split memory access if it can cross memory banks
+STATIC_INLINE void put_long_compatible(uaecptr addr, uae_u32 l)
+{
+	if ((addr & 0xffff) < 0xfffd) {
+		memory_put_long(addr, l);
+	} else if (addr & 1) {
+		memory_put_byte(addr + 0, l >> 24);
+		memory_put_word(addr + 1, l >>  8);
+		memory_put_byte(addr + 3, l >>  0);
+	} else {
+		memory_put_word(addr + 0, l >> 16);
+		memory_put_word(addr + 2, l >>  0);
+	}
+}
+STATIC_INLINE void put_word_compatible(uaecptr addr, uae_u32 w)
+{
+	if ((addr & 0xffff) < 0xffff) {
+		memory_put_word(addr, w);
+	} else {
+		memory_put_byte(addr + 0, w >> 8);
+		memory_put_byte(addr + 1, w >> 0);
+	}
+}
+STATIC_INLINE void put_byte_compatible(uaecptr addr, uae_u32 b)
+{
+	memory_put_byte(addr, b);
+}
+
+
 STATIC_INLINE void put_long_jit(uaecptr addr, uae_u32 l)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_write_flag;
 #endif
-	bank->lput(addr, l);
+	memory_put_long(addr, l);
 }
 STATIC_INLINE void put_word_jit(uaecptr addr, uae_u32 l)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_write_flag;
 #endif
-	bank->wput(addr, l);
+	memory_put_word(addr, l);
 }
 STATIC_INLINE void put_byte_jit(uaecptr addr, uae_u32 l)
 {
-	addrbank *bank = &get_mem_bank(addr);
 #ifdef JIT
+	addrbank *bank = &get_mem_bank(addr);
 	special_mem |= bank->jit_write_flag;
 #endif
-	bank->bput(addr, l);
+	memory_put_byte(addr, l);
 }
 
 /*
@@ -699,14 +782,18 @@ STATIC_INLINE void put_pointer (uaecptr addr, void *v)
 # endif
 #endif
 
+bool real_address_allowed(void);
+uae_u8 *memory_get_real_address(uaecptr);
+int memory_valid_address(uaecptr, uae_u32);
+
 STATIC_INLINE uae_u8 *get_real_address (uaecptr addr)
 {
-	return get_mem_bank (addr).xlateaddr(addr);
+	return memory_get_real_address(addr);
 }
 
 STATIC_INLINE int valid_address (uaecptr addr, uae_u32 size)
 {
-	return get_mem_bank (addr).check(addr, size);
+	return memory_valid_address(addr, size);
 }
 
 STATIC_INLINE void put_quad_host(void *addr, uae_u64 v)
