@@ -7,13 +7,14 @@
   debuginfo.c - functions needed to show info about the atari HW & OS
    components and "lock" that info to be shown on entering the debugger.
 */
-const char DebugInfo_fileid[] = "Hatari debuginfo.c : " __DATE__ " " __TIME__;
+const char DebugInfo_fileid[] = "Hatari debuginfo.c";
 
 #include <stdio.h>
 #include <assert.h>
 #include <ctype.h>
 
 #include "main.h"
+#include "acia.h"
 #include "bios.h"
 #include "blitter.h"
 #include "configuration.h"
@@ -23,16 +24,22 @@ const char DebugInfo_fileid[] = "Hatari debuginfo.c : " __DATE__ " " __TIME__;
 #include "debugdsp.h"
 #include "debugui.h"
 #include "debug_priv.h"
+#include "dmaSnd.h"
 #include "dsp.h"
 #include "evaluate.h"
 #include "file.h"
 #include "gemdos.h"
 #include "history.h"
 #include "ioMem.h"
+#include "ikbd.h"
 #include "m68000.h"
+#include "mfp.h"
+#include "nvram.h"
 #include "psg.h"
+#include "rtc.h"
 #include "stMemory.h"
 #include "tos.h"
+#include "scc.h"
 #include "screen.h"
 #include "vdi.h"
 #include "video.h"
@@ -491,7 +498,7 @@ static void DebugInfo_RegAddr(FILE *fp, Uint32 arg)
 	Uint32 *reg32, regvalue, mask;
 	char cmdbuf[12], addrbuf[6];
 	char *argv[] = { cmdbuf, addrbuf };
-	
+
 	regname[0] = (arg>>24)&0xff;
 	regname[1] = (arg>>16)&0xff;
 	regname[2] = '\0';
@@ -570,7 +577,7 @@ static Uint32 DebugInfo_RegAddrArgs(int argc, char *argv[])
 		fprintf(stderr, "ERROR: invalid address/data register '%s'!\n", argv[1]);
 		return 0;
 	}
-	
+
 	value |= argv[1][0] << 24;
 	value |= argv[1][1] << 16;
 	value &= 0xffff00ff;
@@ -648,11 +655,12 @@ static const struct {
 	/* if overlaps with other functionality, list only for lock command */
 	bool lock;
 	const char *name;
-	void (*func)(FILE *fp, Uint32 arg);
+	info_func_t func;
 	/* convert args in argv into single Uint32 for func */
 	Uint32 (*args)(int argc, char *argv[]);
 	const char *info;
 } infotable[] = {
+	{ false,"acia",      ACIA_Info,            NULL, "Show ACIA register contents" },
 	{ false,"aes",       AES_Info,             NULL, "Show AES vector contents (with <value>, show opcodes)" },
 	{ false,"basepage",  DebugInfo_Basepage,   NULL, "Show program basepage contents at given <address>" },
 	{ false,"bios",      Bios_Info,            NULL, "Show BIOS opcodes" },
@@ -661,20 +669,26 @@ static const struct {
 	{ false,"crossbar",  Crossbar_Info,        NULL, "Show Falcon Crossbar register contents" },
 	{ true, "default",   DebugInfo_Default,    NULL, "Show default debugger entry information" },
 	{ true, "disasm",    DebugInfo_CpuDisAsm,  NULL, "Disasm CPU from PC or given <address>" },
+	{ false,"dmasnd",    DmaSnd_Info,          NULL, "Show Sound DMA / LMC register contents" },
 #if ENABLE_DSP_EMU
 	{ false, "dsp",      DSP_Info,             NULL, "Show misc. DSP core info (stack etc)" },
 	{ true, "dspdisasm", DebugInfo_DspDisAsm,  NULL, "Disasm DSP from given <address>" },
 	{ true, "dspmemdump",DebugInfo_DspMemDump, DebugInfo_DspMemArgs, "Dump DSP memory from given <space> <address>" },
 	{ true, "dspregs",   DebugInfo_DspRegister,NULL, "Show DSP register contents" },
 #endif
-	{ false, "dta",      DebugInfo_DTA,       NULL, "Show current [or given] DTA information" },
+	{ false, "dta",      DebugInfo_DTA,       NULL, "Show current [or given] DTA information" },
 	{ true, "file",      DebugInfo_FileParse, DebugInfo_FileArgs, "Parse commands from given debugger input <file>" },
 	{ false,"gemdos",    GemDOS_Info,          NULL, "Show GEMDOS HDD emu information (with <value>, show opcodes)" },
 	{ true, "history",   History_Show,         NULL, "Show history of last <count> instructions" },
+	{ false,"ikbd",      IKBD_Info,            NULL, "Show IKBD (SCI) register contents" },
 	{ true, "memdump",   DebugInfo_CpuMemDump, NULL, "Dump CPU memory from given <address>" },
+	{ false,"mfp",       MFP_Info,             NULL, "Show MFP register contents" },
+	{ false,"nvram",     NvRam_Info,           NULL, "Show (TT/Falcon) NVRAM contents" },
 	{ false,"osheader",  DebugInfo_OSHeader,   NULL, "Show TOS OS header contents" },
 	{ true, "regaddr",   DebugInfo_RegAddr, DebugInfo_RegAddrArgs, "Show <disasm|memdump> from CPU/DSP address pointed by <register>" },
 	{ true, "registers", DebugInfo_CpuRegister,NULL, "Show CPU register contents" },
+	{ false,"rtc",       Rtc_Info,             NULL, "Show (Mega ST/STE) RTC register contents" },
+	{ false,"scc",       SCC_Info,             NULL, "Show SCC register contents" },
 	{ false,"vdi",       VDI_Info,             NULL, "Show VDI vector contents (with <value>, show opcodes)" },
 	{ false,"videl",     Videl_Info,           NULL, "Show Falcon Videl register contents" },
 	{ false,"video",     Video_Info,           NULL, "Show Video information" },
@@ -682,7 +696,7 @@ static const struct {
 	{ false,"ym",        PSG_Info,             NULL, "Show YM-2149 register contents" },
 };
 
-static int LockedFunction = 6; /* index for the "default" function */
+static int LockedFunction = 7; /* index for the "default" function */
 static Uint32 LockedArgument;
 
 /**
@@ -694,6 +708,19 @@ void DebugInfo_ShowSessionInfo(void)
 	infotable[LockedFunction].func(stderr, LockedArgument);
 }
 
+/**
+ * Return info function matching the given name, or NULL for no match
+ */
+info_func_t DebugInfo_GetInfoFunc(const char *name)
+{
+	int i;
+	for (i = 0; i < ARRAY_SIZE(infotable); i++) {
+		if (strcmp(name, infotable[i].name) == 0) {
+			return infotable[i].func;
+		}
+	}
+	return NULL;
+}
 
 /**
  * Readline match callback for info subcommand name completion.
@@ -704,7 +731,7 @@ static char *DebugInfo_Match(const char *text, int state, bool lock)
 {
 	static int i, len;
 	const char *name;
-	
+
 	if (!state) {
 		/* first match */
 		len = strlen(text);
@@ -743,7 +770,7 @@ int DebugInfo_Command(int nArgc, char *psArgs[])
 
 	sub = -1;
 	if (nArgc > 1) {
-		cmd = psArgs[1];		
+		cmd = psArgs[1];
 		/* which subcommand? */
 		for (i = 0; i < ARRAY_SIZE(infotable); i++) {
 			if (strcmp(cmd, infotable[i].name) == 0) {
@@ -768,7 +795,7 @@ int DebugInfo_Command(int nArgc, char *psArgs[])
 	}
 
 	lock = (strcmp(psArgs[0], "lock") == 0);
-	
+
 	if (sub < 0 || !ok) {
 		/* no subcommand or something wrong with value, show info */
 		fprintf(stderr, "%s subcommands are:\n", psArgs[0]);
