@@ -1,14 +1,14 @@
 /*
  * Hatari - profile.c
- * 
- * Copyright (C) 2010-2015 by Eero Tamminen
+ *
+ * Copyright (C) 2010-2019 by Eero Tamminen
  *
  * This file is distributed under the GNU General Public License, version 2
  * or at your option any later version. Read the file gpl.txt for details.
  *
  * profile.c - profile caller info handling and debugger parsing functions
  */
-const char Profile_fileid[] = "Hatari profile.c : " __DATE__ " " __TIME__;
+const char Profile_fileid[] = "Hatari profile.c";
 
 #include <stdio.h>
 #include <assert.h>
@@ -23,6 +23,8 @@ const char Profile_fileid[] = "Hatari profile.c : " __DATE__ " " __TIME__;
 #include "symbols.h"
 #include "profile.h"
 #include "profile_priv.h"
+#include "m68000.h"
+#include "dsp.h"
 
 profile_loop_t profile_loop;
 
@@ -111,11 +113,8 @@ static void output_caller_info(FILE *fp, caller_t *info, Uint32 *typeaddr)
 	fputs(", ", fp);
 }
 
-/*
- * Show collected CPU/DSP callee/caller information.
- *
- * Hint: As caller info list is based on number of loaded symbols,
- * load only text symbols to save memory & make things faster...
+/**
+ * Show collected CPU/DSP callee/caller information
  */
 void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (*addr2name)(Uint32, Uint64 *))
 {
@@ -165,7 +164,7 @@ void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (
 			countissues++;
 		}
 		if (typeaddr) {
-			fprintf(stderr, "WARNING: different types of calls (at least) from 0x%x (to 0x%x),\n\t has its codechanged during profiling?\n",
+			fprintf(stderr, "WARNING: different types of calls (at least) from 0x%x (to 0x%x),\n\t has its code changed during profiling?\n",
 				typeaddr, callsite->addr);
 		}
 	}
@@ -235,11 +234,13 @@ static void add_callee_cost(callee_t *callsite, callstack_t *stack)
 	assert(0);
 }
 
-
+/**
+ * Add new caller or updated earlier caller stats for call site
+ */
 static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t flag)
 {
+	int i, count, oldcount;
 	caller_t *info;
-	int i, count;
 
 	/* need to store real call addresses as symbols can change
 	 * after profiling has been stopped
@@ -258,21 +259,24 @@ static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t
 	}
 	/* how many caller slots are currently allocated? */
 	count = callsite->count;
+	oldcount = 0;
 	for (;;) {
-		for (i = 0; i < count; i++, info++) {
+		for (i = oldcount; i < count; i++, info++) {
 			if (info->addr == prev_pc) {
+				/* increment caller */
 				info->flags |= flag;
 				info->calls++;
 				return;
 			}
 			if (!info->addr) {
-				/* empty slot */
+				/* add caller to empty slot */
 				info->addr = prev_pc;
 				info->flags |= flag;
 				info->calls = 1;
 				return;
 			}
 		}
+		oldcount = count;
 		/* not enough, double caller slots */
 		count *= 2;
 		info = realloc(callsite->callers, count * sizeof(*info));
@@ -280,14 +284,15 @@ static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t
 			fprintf(stderr, "ERROR: caller info alloc failed!\n");
 			return;
 		}
-		memset(info + callsite->count, 0, callsite->count * sizeof(*info));
 		callsite->callers = info;
 		callsite->count = count;
+		info = info + oldcount;
+		memset(info, 0, oldcount * sizeof(*info));
 	}
 }
 
 /**
- * Add information about called symbol, and if it was subroutine
+ * Add information about the called symbol, and if it was a subroutine
  * call, add it to stack of functions which total costs are tracked.
  * callinfo.return_pc needs to be set before invoking this if the call
  * is of type CALL_SUBROUTINE.
@@ -304,8 +309,8 @@ void Profile_CallStart(int idx, callinfo_t *callinfo, Uint32 prev_pc, calltype_t
 
 	add_caller(callinfo->site + idx, pc, prev_pc, flag);
 
-	/* subroutine call which will return? */
-	if (flag != CALL_SUBROUTINE) {
+	/* subroutine/exception call i.e. one which will return? */
+	if (flag != CALL_SUBROUTINE && flag != CALL_EXCEPTION) {
 		/* no, some other call type */
 		return;
 	}
@@ -402,50 +407,142 @@ Uint32 Profile_CallEnd(callinfo_t *callinfo, counters_t *totalcost)
 	return stack->caller_addr;
 }
 
+
 /**
- * Add costs to all functions still in call stack
+ * Add costs to all functions still in call stack and print their names
+ *
+ * Diagram of variables involved in 2 functions deep call stack:
+ *
+ *   (caller_addr 1)  bsr symbol1  -1->  symbol1     (callee_addr 1)
+ *      (ret_addr 1)  <instr>     <-.    ...
+ *                                  |    bsr symbol2 (caller_addr 2)  -2->  symbol2 (callee_addr 2)
+ *                                  1    <instr>     (ret_addr 2)    <-.    ...
+ *                                  |    ...                           2    ... (PC)
+ *                                  '-   rts                           '-   rts
+ *
+ * When one wants to match callee_addr (= symbol) to caller_addr (= which
+ * place in that function called further functions), it's best to traverse
+ * stack from last item to top, as these items are in following call stack
+ * items, not in same one.
  */
-void Profile_FinalizeCalls(callinfo_t *callinfo, counters_t *totalcost, const char* (*get_symbol)(Uint32, symtype_t))
+void Profile_FinalizeCalls(Uint32 pc, callinfo_t *callinfo, counters_t *totalcost,
+			   const char* (*get_symbol)(Uint32, symtype_t),
+			   const char* (*get_caller)(Uint32*))
 {
-	Uint32 addr;
+	const char *sym, *caller;
+	Uint32 sym_addr, caller_addr;
+	int i, lines, offset;
+	bool dots;
+	char sign;
+
 	if (!callinfo->depth) {
 		return;
 	}
 	fprintf(stderr, "Finalizing costs for %d non-returned functions:\n", callinfo->depth);
+
+	i = 0;
+	dots = false;
+	caller_addr = pc;
+	lines = ConfigureParams.Debugger.nBacktraceLines;
 	while (callinfo->depth > 0) {
+
+		/* finalize & decrease callinfo->depth */
 		Profile_CallEnd(callinfo, totalcost);
-		addr = callinfo->stack[callinfo->depth].callee_addr;
-		fprintf(stderr, "- 0x%x: %s (return = 0x%x)\n",
-			addr, get_symbol(addr, SYMTYPE_TEXT),
-			callinfo->stack[callinfo->depth].ret_addr);
+		if (++i > lines && lines > 0) {
+			continue;
+		}
+
+		/* Skip showing middle part of a long callstack as messed
+		 * callstacks could be thousands of frames deep...
+		 */
+		if (i >= 32 && callinfo->depth > 32) {
+			if (!dots) {
+				fprintf(stderr, "- ...\n");
+				dots = true;
+			}
+		} else {
+			sym_addr = callinfo->stack[callinfo->depth].callee_addr;
+			sym = get_symbol(sym_addr, SYMTYPE_TEXT);
+
+			if (sym) {
+				offset = caller_addr - sym_addr;
+				sign = offset >= 0 ? '+' : '-';
+				fprintf(stderr, "- %d. 0x%06x: %s %c0x%x",
+					i, caller_addr, sym, sign, abs(offset));
+			} else {
+				fprintf(stderr, "- %d. 0x%06x", i, caller_addr);
+			}
+
+			sym_addr = caller_addr;
+			caller = get_caller(&sym_addr);
+			if (caller && caller != sym) {
+				offset = caller_addr - sym_addr;
+				fprintf(stderr, " (%s +0x%x)\n",
+					caller, abs(offset));
+			} else {
+				fprintf(stderr, "\n");
+			}
+		}
+		caller_addr = callinfo->stack[callinfo->depth].caller_addr;
 	}
 }
 
 /**
- * Show current profile stack
+ * Show current profile call stack, up to configured max backtrace depth
  */
 static void Profile_ShowStack(bool forDsp)
 {
-	int i;
-	Uint32 addr;
+	const char *sym, *caller;
+	const char *(*get_caller)(Uint32*);
+	const char *(*get_symbol)(Uint32, symtype_t);
+	Uint32 sym_addr, caller_addr;
+	int i, offset, depth, top;
 	callinfo_t *callinfo;
-	const char* (*get_symbol)(Uint32, symtype_t);
 
 	if (forDsp) {
-		Profile_DspGetCallinfo(&callinfo, &get_symbol);
+		Profile_DspGetCallinfo(&callinfo, &get_caller, &get_symbol);
+		caller_addr = DSP_GetPC();
 	} else {
-		Profile_CpuGetCallinfo(&callinfo, &get_symbol);
+		Profile_CpuGetCallinfo(&callinfo, &get_caller, &get_symbol);
+		caller_addr = M68000_GetPC();
 	}
 	if (!callinfo->depth) {
 		fprintf(stderr, "Empty stack.\n");
 		return;
 	}
 
-	for (i = 0; i < callinfo->depth; i++) {
-		addr = callinfo->stack[i].callee_addr;
-		fprintf(stderr, "- 0x%x: %s (return = 0x%x)\n",
-			addr, get_symbol(addr, SYMTYPE_TEXT),
-			callinfo->stack[i].ret_addr);
+	depth = callinfo->depth;
+	top = ConfigureParams.Debugger.nBacktraceLines;
+	if (top > 0 && top < depth) {
+		top = depth - top;
+	} else {
+		top = 0;
+	}
+	i = 0;
+	while (depth-- > top) {
+		i++;
+		sym_addr = callinfo->stack[depth].callee_addr;
+		offset = caller_addr - sym_addr;
+
+		sym = get_symbol(sym_addr, SYMTYPE_TEXT);
+		if (sym) {
+			char sign = offset >= 0 ? '+' : '-';
+			fprintf(stderr, "- %d. 0x%06x: %s %c0x%x",
+				i, caller_addr, sym, sign, abs(offset));
+		} else {
+			fprintf(stderr, "- %d. 0x%06x", i, caller_addr);
+		}
+
+		sym_addr = caller_addr;
+		caller = get_caller(&sym_addr);
+		if (caller && caller != sym) {
+			offset = caller_addr - sym_addr;
+			fprintf(stderr, " (%s +0x%x)\n",
+				caller, abs(offset));
+		} else {
+			fprintf(stderr, "\n");
+		}
+		caller_addr = callinfo->stack[depth].caller_addr;
 	}
 }
 
@@ -695,7 +792,7 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 	} else if (strcmp(psArgs[1], "off") == 0) {
 		*enabled = false;
 		fprintf(stderr, "Profiling disabled.\n");
-	
+
 	} else if (strcmp(psArgs[1], "stats") == 0) {
 		if (bForDsp) {
 			Profile_DspShowStats();
