@@ -1,14 +1,14 @@
 /*
  * Hatari - profilecpu.c
  * 
- * Copyright (C) 2010-2015 by Eero Tamminen
+ * Copyright (C) 2010-2019 by Eero Tamminen
  *
  * This file is distributed under the GNU General Public License, version 2
  * or at your option any later version. Read the file gpl.txt for details.
  *
  * profilecpu.c - functions for profiling CPU and showing the results.
  */
-const char Profilecpu_fileid[] = "Hatari profilecpu.c : " __DATE__ " " __TIME__;
+const char Profilecpu_fileid[] = "Hatari profilecpu.c";
 
 #include <stdio.h>
 #include <inttypes.h>
@@ -107,8 +107,21 @@ static struct {
 	bool enabled;         /* true when profiling enabled */
 } cpu_profile;
 
-/* special hack for EmuTOS */
-static Uint32 etos_switcher;
+/* full counts for warnings that are printed without rate-limiting */
+typedef struct {
+	int odd;
+	int address;
+	int returns;
+	int multireturn;
+	int prevpc;
+	int largevalue;
+	int opfamily;
+	int zerocycles;
+} cpu_warnings_t;
+static cpu_warnings_t cpu_warnings;
+
+#define MAX_SHOW_COUNT	8
+#define MAX_MULTI_RETURN 1
 
 
 /* ------------------ CPU profile address mapping ----------------- */
@@ -119,7 +132,12 @@ static Uint32 etos_switcher;
 static inline Uint32 address2index(Uint32 pc)
 {
 	if (unlikely(pc & 1)) {
-		fprintf(stderr, "WARNING: odd CPU profile instruction address 0x%x!\n", pc);
+		if (++cpu_warnings.odd <= MAX_SHOW_COUNT) {
+			fprintf(stderr, "WARNING: odd CPU profile instruction address 0x%x!\n", pc);
+			if (cpu_warnings.odd == MAX_SHOW_COUNT) {
+				fprintf(stderr, "Further warnings won't be shown.\n");
+			}
+		}
 #if DEBUG
 		skip_assert = true;
 		DebugUI(REASON_CPU_EXCEPTION);
@@ -147,7 +165,12 @@ static inline Uint32 address2index(Uint32 pc)
 		pc += STRamEnd + TosSize + CART_SIZE - TTRAM_START;
 #endif
 	} else {
-		fprintf(stderr, "WARNING: 'invalid' CPU PC profile instruction address 0x%x!\n", pc);
+		if (++cpu_warnings.address <= MAX_SHOW_COUNT) {
+			fprintf(stderr, "WARNING: 'invalid' CPU PC profile instruction address 0x%x!\n", pc);
+			if (cpu_warnings.address == MAX_SHOW_COUNT) {
+				fprintf(stderr, "Further warnings won't be shown.\n");
+			}
+		}
 		/* extra entry at end is reserved for invalid PC values */
 		pc = STRamEnd + TosSize + CART_SIZE;
 #if DEBUG
@@ -200,23 +223,47 @@ static Uint32 index2address(Uint32 idx)
 /* ------------------ CPU profile results ----------------- */
 
 /**
- * Write string containing CPU cache stats, cycles, count, count percentage
- * for given address to provided buffer.
- *
- * Return true if data was available and non-zero, false otherwise.
+ * Return true if there's profile data for given address, false otherwise
  */
-bool Profile_CpuAddressDataStr(char *buffer, size_t maxlen, Uint32 addr)
+bool Profile_CpuAddr_HasData(Uint32 addr)
 {
 	cpu_profile_item_t *item;
-	float percentage;
 	Uint32 idx;
 
-	assert(buffer && maxlen);
 	if (!cpu_profile.data) {
 		return false;
 	}
 	idx = address2index(addr);
 	item = &(cpu_profile.data[idx]);
+	if (!item->count) {
+		return false;
+	}
+	return true;
+}
+
+/**
+ * Write string containing CPU cache stats, cycles, count, count percentage
+ * for given address to provided buffer.
+ *
+ * Return zero if there's no profiling data for given address,
+ * otherwise return the number of bytes consumed from the given buffer.
+ */
+int Profile_CpuAddr_DataStr(char *buffer, int maxlen, Uint32 addr)
+{
+	cpu_profile_item_t *item;
+	float percentage;
+	Uint32 idx;
+	int count;
+
+	assert(buffer && maxlen > 0);
+	if (!cpu_profile.data) {
+		return 0;
+	}
+	idx = address2index(addr);
+	item = &(cpu_profile.data[idx]);
+	if (!item->count) {
+		return 0;
+	}
 
 	if (cpu_profile.all.count) {
 		percentage = 100.0 * item->count / cpu_profile.all.count;
@@ -224,16 +271,57 @@ bool Profile_CpuAddressDataStr(char *buffer, size_t maxlen, Uint32 addr)
 		percentage = 0.0;
 	}
 #if DEBUG_CACHE
-	snprintf(buffer, maxlen, "%5.2f%% (%u, %u, %u, %u, %u, %u)",
-		 percentage, item->count, item->cycles,
-		 item->i_hits, item->i_misses,
-		 item->d_hits, item->d_misses);
+	count = snprintf(buffer, maxlen, "%5.2f%% (%u, %u, %u, %u, %u, %u)",
+			 percentage, item->count, item->cycles,
+			 item->i_hits, item->i_misses,
+			 item->d_hits, item->d_misses);
 #else
-	snprintf(buffer, maxlen, "%5.2f%% (%u, %u, %u, %u)",
-		 percentage, item->count, item->cycles,
-		 item->i_misses, item->d_hits);
+	count = snprintf(buffer, maxlen, "%5.2f%% (%u, %u, %u, %u)",
+			 percentage, item->count, item->cycles,
+			 item->i_misses, item->d_hits);
 #endif
-	return (item->count > 0);
+	if (count >= maxlen) {
+		/* truncated by (count - maxlen) amount */
+		return maxlen;
+	}
+	return count;
+}
+
+/**
+ * Show CPU profiling warning counts
+ */
+static void show_cpu_warnings(void)
+{
+	cpu_warnings_t warnings;
+	memset(&warnings, 0, sizeof(warnings));
+	if (memcmp(&cpu_warnings, &warnings, sizeof(warnings)) == 0) {
+		return;
+	}
+	fprintf(stderr, "\nCPU profiling warning counts:\n");
+	if (cpu_warnings.odd) {
+		fprintf(stderr, "- Odd PC addresses: %d\n", cpu_warnings.odd);
+	}
+	if (cpu_warnings.address) {
+		fprintf(stderr, "- Unmapped PC addresses: %d\n", cpu_warnings.address);
+	}
+	if (cpu_warnings.opfamily) {
+		fprintf(stderr, "- Unrecognized (zero) opcode families: %d\n", cpu_warnings.opfamily);
+	}
+	if (cpu_warnings.returns) {
+		fprintf(stderr, "- Subroutine calls didn't return through RTS etc: %d\n", cpu_warnings.returns);
+	}
+	if (cpu_warnings.multireturn > MAX_MULTI_RETURN) {
+		fprintf(stderr, "- Subroutine calls returned (at max) through %d stack frames\n", cpu_warnings.multireturn);
+	}
+	if (cpu_warnings.prevpc) {
+		fprintf(stderr, "- Undefined PC value for tracked address callers: %d\n", cpu_warnings.prevpc);
+	}
+	if (cpu_warnings.largevalue) {
+		fprintf(stderr, "- Unexpectedly large cycles count or cache hit/miss values: %d\n", cpu_warnings.largevalue);
+	}
+	if (cpu_warnings.zerocycles) {
+		fprintf(stderr, "- Successive instructions with zero cycles: %d\n", cpu_warnings.zerocycles);
+	}
 }
 
 /**
@@ -276,7 +364,7 @@ static void show_cpu_area_stats(profile_area_t *area)
 
 
 /**
- * show CPU area (RAM, ROM, TOS) specific statistics.
+ * show CPU area (RAM, ROM, TOS) specific statistics and error counts.
  */
 void Profile_CpuShowStats(void)
 {
@@ -293,9 +381,10 @@ void Profile_CpuShowStats(void)
 		fprintf(stderr, "TT-RAM (0x%X-%X):\n", TTRAM_START, TTRAM_START + 1024*ConfigureParams.Memory.TTRamSize_KB);
 		show_cpu_area_stats(&cpu_profile.ttram);
 	}
-
 	fprintf(stderr, "\n= %.5fs\n",
 		(double)cpu_profile.all.cycles / MachineClocks.CPU_Freq_Emul);
+
+	show_cpu_warnings();
 }
 
 #if ENABLE_WINUAE_CPU
@@ -691,7 +780,7 @@ void Profile_CpuShowCounts(int show, bool only_symbols)
 
 	leave_instruction_column(oldcols);
 
-	printf("addr:\t\tcount:\t\tsymbol:\n");
+	printf("addr:        %%:   count:  symbol:                    disassembly:\n");
 	for (end = sort_arr + active; sort_arr < end; sort_arr++) {
 
 		addr = index2address(*sort_arr);
@@ -701,9 +790,9 @@ void Profile_CpuShowCounts(int show, bool only_symbols)
 		}
 		count = data[*sort_arr].count;
 		percentage = 100.0*count/cpu_profile.all.count;
-		printf("0x%06x\t%5.2f%%\t%d\t%s%s\t",
+		printf("0x%06x %6.2f %8d  %-26s %s",
 		       addr, percentage, count, name,
-		       count == MAX_CPU_PROFILE_VALUE ? " (OVERFLOW)" : "");
+		       count == MAX_CPU_PROFILE_VALUE ? "(OVERFLOW) " : "");
 		Disasm(stdout, addr, &nextpc, 1);
 
 		matched++;
@@ -786,6 +875,8 @@ bool Profile_CpuStart(void)
 	}
 	/* zero everything */
 	memset(&cpu_profile, 0, sizeof(cpu_profile));
+	memset(&cpu_warnings, 0, sizeof(cpu_warnings));
+	cpu_warnings.multireturn = MAX_MULTI_RETURN;
 
 	/* Shouldn't change within same debug session */
 	size = (STRamEnd + CART_SIZE + TosSize) / 2;
@@ -805,13 +896,6 @@ bool Profile_CpuStart(void)
 
 	Profile_AllocCallinfo(&(cpu_callinfo), Symbols_CpuCodeCount(), "CPU");
 
-	/* special hack for EmuTOS */
-	etos_switcher = PC_UNDEFINED;
-	if (cpu_callinfo.sites && bIsEmuTOS &&
-	    (!Symbols_GetCpuAddress(SYMTYPE_TEXT, "_switchto", &etos_switcher) || etos_switcher < TosAddress)) {
-		etos_switcher = PC_UNDEFINED;
-	}
-
 	/* reset cache stats (CPU emulation doesn't do that) */
 	CpuInstruction.D_Cache_hit = 0;
 	CpuInstruction.I_Cache_hit = 0;
@@ -891,6 +975,31 @@ static calltype_t cpu_opcode_type(int family, Uint32 prev_pc, Uint32 pc)
 	return CALL_UNKNOWN;
 }
 
+/**
+ * Check callstack to see if return was for an earlier subroutine
+ * call higher in the stack.
+ *
+ * This can happen when e.g. OS task switchers manipulate stack to
+ * convert subroutine return (RTS) into a jump to another function.
+ * I.e. one of the returns in callstack will be skipped.
+ *
+ * Returns number of frames to finish/end.
+ */
+static int returned_frames(callinfo_t *callinfo, Uint32 pc)
+{
+	int frames, depth;
+	Uint32 return_pc;
+
+	depth = callinfo->depth;
+	for (frames = 1; --depth >= 0; frames++) {
+		return_pc = callinfo->stack[depth].ret_addr;
+		if (pc == return_pc) {
+			return frames;
+		}
+	}
+	return 0;
+}
+
 /**
  * If call tracking is enabled (there are symbols), collect
  * information about subroutine and other calls, and their costs.
@@ -902,7 +1011,7 @@ static calltype_t cpu_opcode_type(int family, Uint32 prev_pc, Uint32 pc)
 static void collect_calls(Uint32 pc, counters_t *counters)
 {
 	calltype_t flag;
-	int idx, family;
+	int frames, idx, family;
 	Uint32 prev_pc, caller_pc;
 
 	family = cpu_profile.prev_family;
@@ -912,58 +1021,60 @@ static void collect_calls(Uint32 pc, counters_t *counters)
 	cpu_callinfo.prev_pc = pc;
 	caller_pc = PC_UNDEFINED;
 
-	/* address is return address for last subroutine call? */
-	if (unlikely(pc == cpu_callinfo.return_pc) && likely(cpu_callinfo.depth)) {
-
-		flag = cpu_opcode_type(family, prev_pc, pc);
-		/* previous address can be exception return (e.g. RTE) instead of RTS,
-		 * if exception occurred right after returning from subroutine call.
+	/* check opcode first as return frame check can be slow with deep call stacks */
+	flag = cpu_opcode_type(family, prev_pc, pc);
+	if (unlikely(flag == CALL_SUBRETURN || flag == CALL_EXCRETURN)) {
+		/* is address a return address for *any* of the previous subroutine calls? */
+		frames = returned_frames(&cpu_callinfo, pc);
+		if (frames) {
+			if (unlikely(frames > cpu_warnings.multireturn)) {
+				fprintf(stderr, "WARNING: subroutine call returned through %d stack frames: 0x%x -> 0x%x!\n",
+					frames, prev_pc, pc);
+				cpu_warnings.multireturn = frames;
+			}
+			/* unwind callstack & update costs */
+			while (frames-- > 0) {
+				caller_pc = Profile_CallEnd(&cpu_callinfo, counters);
+			}
+		}
+	} else if (unlikely(pc == cpu_callinfo.return_pc)) {
+		/* return address, but not due to return, e.g. because
+		 * there was a jsr or jump to return address.  Checked
+		 * only for last return
 		 */
-		if (likely(flag == CALL_SUBRETURN || flag == CALL_EXCRETURN)) {
-			caller_pc = Profile_CallEnd(&cpu_callinfo, counters);
-		} else {
-#if DEBUG
-			/* although at return address, it didn't return yet,
-			 * e.g. because there was a jsr or jump to return address
-			 */
+		if (++cpu_warnings.returns <= MAX_SHOW_COUNT) {
 			Uint32 nextpc;
-			fprintf(stderr, "WARNING: subroutine call returned 0x%x -> 0x%x, not through RTS!\n", prev_pc, pc);
+			fprintf(stderr, "WARNING: subroutine call returned 0x%x -> 0x%x, not through RTS etc!\n", prev_pc, pc);
 			Disasm(stderr, prev_pc, &nextpc, 1);
-#endif
+			if (cpu_warnings.returns == MAX_SHOW_COUNT) {
+				fprintf(stderr, "Further warnings won't be shown.\n");
+			}
 		}
-		/* next address might be another symbol, so need to fall through */
 	}
 
 	/* address is one which we're tracking? */
 	idx = Symbols_GetCpuCodeIndex(pc);
 	if (unlikely(idx >= 0)) {
-
-		flag = cpu_opcode_type(family, prev_pc, pc);
-		if (flag == CALL_SUBROUTINE || flag == CALL_EXCEPTION) {
-			/* special HACK for for EmuTOS AES switcher which
-			 * changes stack content to remove itself from call
-			 * stack and uses RTS for subroutine *calls*, not
-			 * for returning from them.
-			 *
-			 * It wouldn't be reliable to detect calls from it,
-			 * so I'm making call *to* it show up as branch, to
-			 * keep callstack depth correct.
-			 */
-			if (unlikely(pc == etos_switcher)) {
-				flag = CALL_BRANCH;
-			} else if (unlikely(prev_pc == PC_UNDEFINED)) {
+		/* normal subroutine / exception call? */
+		if (likely(flag == CALL_SUBROUTINE || flag == CALL_EXCEPTION)) {
+			if (unlikely(prev_pc == PC_UNDEFINED)) {
 				/* if first profiled instruction
 				 * is subroutine call, it doesn't have
 				 * valid prev_pc value stored
 				 */
 				cpu_callinfo.return_pc = PC_UNDEFINED;
-				fprintf(stderr, "WARNING: previous PC for tracked address 0x%d is undefined!\n", pc);
+				if (++cpu_warnings.prevpc <= MAX_SHOW_COUNT) {
+					fprintf(stderr, "WARNING: previous PC for tracked address 0x%d is undefined!\n", pc);
+					if (cpu_warnings.prevpc == MAX_SHOW_COUNT) {
+						fprintf(stderr, "Further warnings won't be shown.\n");
+					}
+				}
 #if DEBUG
 				skip_assert = true;
 				DebugUI(REASON_CPU_EXCEPTION);
 #endif
 			} else {
-				/* slow! */
+				/* return is to next instruction (slow!) */
 				cpu_callinfo.return_pc = Disasm_GetNextPC(prev_pc);
 			}
 		} else if (caller_pc != PC_UNDEFINED) {
@@ -1000,10 +1111,15 @@ static void log_last_loop(void)
  */
 static Uint32 warn_too_large(const char *name, const int value, const int limit, const Uint32 prev_pc, const Uint32 pc)
 {
-	Uint32 nextpc;
-	fprintf(stderr, "WARNING: unexpected (%d > %d) %s at 0x%x:\n", value, limit - 1, name, pc);
-	Disasm(stderr, prev_pc, &nextpc, 1);
-	Disasm(stderr, pc, &nextpc, 1);
+	if (++cpu_warnings.largevalue <= MAX_SHOW_COUNT) {
+		Uint32 nextpc;
+		fprintf(stderr, "WARNING: unexpected (%d > %d) %s at 0x%x:\n", value, limit - 1, name, pc);
+		Disasm(stderr, prev_pc, &nextpc, 1);
+		Disasm(stderr, pc, &nextpc, 1);
+		if (cpu_warnings.largevalue == MAX_SHOW_COUNT) {
+			fprintf(stderr, "Further warnings won't be shown.\n");
+		}
+	}
 #if DEBUG
 	skip_assert = true;
 	DebugUI(REASON_CPU_EXCEPTION);
@@ -1148,9 +1264,14 @@ void Profile_CpuUpdate(void)
 
 #if DEBUG
 	if (unlikely(OpcodeFamily == 0)) {
-		Uint32 nextpc;
-		fputs("WARNING: instruction opcode family is zero (=i_ILLG) for instruction:\n", stderr);
-		Disasm(stderr, prev_pc, &nextpc, 1);
+		if (++cpu_warnings.opfamily <= MAX_SHOW_COUNT) {
+			Uint32 nextpc;
+			fputs("WARNING: instruction opcode family is zero (=i_ILLG) for instruction:\n", stderr);
+			Disasm(stderr, prev_pc, &nextpc, 1);
+			if (cpu_warnings.opfamily == MAX_SHOW_COUNT) {
+				fprintf(stderr, "Further warnings won't be shown.\n");
+			}
+		}
 	}
 	/* catch too large (and negative) cycles for other than STOP instruction */
 	if (unlikely(cycles > 512 && OpcodeFamily != i_STOP)) {
@@ -1160,10 +1281,15 @@ void Profile_CpuUpdate(void)
 	{
 		static Uint32 prev_cycles = 0, prev_pc2 = 0;
 		if (unlikely(cycles == 0 && prev_cycles == 0)) {
-			Uint32 nextpc;
-			fputs("WARNING: Zero cycles for successive opcodes:\n", stderr);
-			Disasm(stderr, prev_pc2, &nextpc, 1);
-			Disasm(stderr, prev_pc, &nextpc, 1);
+			if (++cpu_warnings.zerocycles <= MAX_SHOW_COUNT) {
+				Uint32 nextpc;
+				fputs("WARNING: Zero cycles for successive opcodes:\n", stderr);
+				Disasm(stderr, prev_pc2, &nextpc, 1);
+				Disasm(stderr, prev_pc, &nextpc, 1);
+				if (cpu_warnings.zerocycles == MAX_SHOW_COUNT) {
+					fprintf(stderr, "Further warnings won't be shown.\n");
+				}
+			}
 		}
 		prev_cycles = cycles;
 		prev_pc2 = prev_pc;
@@ -1209,7 +1335,7 @@ static Uint32 update_area(profile_area_t *area, Uint32 start, Uint32 end)
 	Uint32 addr;
 
 	memset(area, 0, sizeof(profile_area_t));
-	area->lowest = cpu_profile.size;
+	area->lowest = end;
 
 	item = &(cpu_profile.data[start]);
 	for (addr = start; addr < end; addr++, item++) {
@@ -1261,7 +1387,11 @@ void Profile_CpuStop(void)
 	}
 	assert(cpu_profile.size == size);
 
-	Profile_FinalizeCalls(&(cpu_callinfo), &(cpu_profile.all), Symbols_GetByCpuAddress);
+	Profile_FinalizeCalls(M68000_GetPC(),
+			      &(cpu_callinfo),
+			      &(cpu_profile.all),
+			      Symbols_GetByCpuAddress,
+			      Symbols_GetBeforeCpuAddress);
 
 	/* find lowest and highest addresses executed etc */
 	next = update_area(&cpu_profile.ram, 0, STRamEnd/2);
@@ -1335,8 +1465,10 @@ void Profile_CpuGetPointers(bool **enabled, Uint32 **disasm_addr)
 /**
  * Get callinfo & symbol search pointers for stack walking.
  */
-void Profile_CpuGetCallinfo(callinfo_t **callinfo, const char* (**get_symbol)(Uint32, symtype_t))
+void Profile_CpuGetCallinfo(callinfo_t **callinfo, const char* (**get_caller)(Uint32*),
+			    const char* (**get_symbol)(Uint32, symtype_t))
 {
 	*callinfo = &(cpu_callinfo);
+	*get_caller = Symbols_GetBeforeCpuAddress;
 	*get_symbol = Symbols_GetByCpuAddress;
 }
