@@ -93,7 +93,7 @@
 */
 
 
-const char DmaSnd_fileid[] = "Hatari dmaSnd.c : " __DATE__ " " __TIME__;
+const char DmaSnd_fileid[] = "Hatari dmaSnd.c";
 
 #include "main.h"
 #include "audio.h"
@@ -148,6 +148,8 @@ struct dma_s {
 
 	Sint16 FrameLeft;		/* latest values read from the FIFO */
 	Sint16 FrameRight;
+
+	Uint8  XSINT_Signal;		/* Value of the XSINT signal (connected to MFP) */
 };
 
 static Sint64	frameCounter_float = 0;
@@ -220,6 +222,8 @@ static const int DmaSndSampleRates[4] =
 /* Local functions prototypes					*/
 /*--------------------------------------------------------------*/
 
+static void	DmaSnd_Update_XSINT_Line ( Uint8 Bit );
+
 static void	DmaSnd_FIFO_Refill(void);
 static Sint8	DmaSnd_FIFO_PullByte(void);
 static void	DmaSnd_FIFO_SetStereo(void);
@@ -250,6 +254,8 @@ void DmaSnd_Reset(bool bCold)
 	dma.FrameLeft = 0;
 	dma.FrameRight = 0;
 
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );	/* O/LOW=dma sound idle */
+
 	if ( bCold )
 	{
 		/* Microwire has no reset signal, it will keep its values on warm reset */
@@ -282,6 +288,21 @@ void DmaSnd_MemorySnapShot_Capture(bool bSave)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Update the value of the XSINT line ; this line is connected to TAI and to GPIP7
+ * Depending on the transition, this can trigger MFP interrupt for Timer A or for GPIP7
+ *  - Bit is set to 0/LOW when dma sound is idle
+ *  - Bit is set to 1/HIGH when dma sound is playing
+ */
+static void DmaSnd_Update_XSINT_Line ( Uint8 Bit )
+{
+	dma.XSINT_Signal = Bit;
+	MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE7 , Bit );
+	MFP_TimerA_Set_Line_Input ( pMFP_Main , Bit );			/* Update events count / interrupt for timer A if needed */
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function is called on every HBL to ensure the DMA Audio's FIFO
@@ -431,7 +452,11 @@ static void DmaSnd_StartNewFrame(void)
 	{
 		nDmaSoundControl &= ~DMASNDCTRL_PLAY;
 		LOG_TRACE(TRACE_DMASND, "DMA snd stopped because new frame start=end=%x and repeat=off\n", dma.frameStartAddr);
+		return;
 	}
+
+	/* DMA sound play : update XSINT */
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_HIGH );	/* 1/HIGH=dma sound play */
 }
 
 
@@ -440,19 +465,26 @@ static void DmaSnd_StartNewFrame(void)
  * End-of-frame has been reached. Raise interrupts if needed.
  * Returns true if DMA sound processing should be stopped now and false
  * if it continues (DMA PLAYLOOP mode).
+ *
+ * NOTE : on early STE models the XSINT signal was directly connected
+ * to MFP GPIP7 and to Timer A input (for event count mode).
+ * On later revisions, as well as on TT, the signal to Timer A input
+ * is now delayed by 8 shifts using a 74LS164 running at 2 MHz, which
+ * is equivalent to 32 CPU cycles when the CPU runs at 8 MHz.
+ * At the emulation level, we don't take into account this delay of
+ * 32 CPU cycles, as it would add complexity and no program are known
+ * so far to require this delay.
  */
 static inline int DmaSnd_EndOfFrameReached(void)
 {
 	LOG_TRACE(TRACE_DMASND, "DMA snd end of frame\n");
 
-	/* Raise end-of-frame interrupts (MFP-i7 and Time-A) */
-	MFP_InputOnChannel ( MFP_INT_GPIP7 , 0 );
-	if (MFP_TACR == 0x08)       /* Is timer A in Event Count mode? */
-		MFP_TimerA_EventCount_Interrupt();
+	/* DMA sound idle : update XSINT */
+	DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );		/* O/LOW=dma sound idle */
 
 	if (nDmaSoundControl & DMASNDCTRL_PLAYLOOP)
 	{
-		DmaSnd_StartNewFrame();
+		DmaSnd_StartNewFrame();					/* update XSINT */
 	}
 	else
 	{
@@ -756,11 +788,12 @@ void DmaSnd_SoundControl_WriteWord(void)
 		LOG_TRACE(TRACE_DMASND, "DMA snd control write: starting dma sound output\n");
 		DmaInitSample = true;
 		frameCounter_float = 0;
-		DmaSnd_StartNewFrame();			/* this can clear DMASNDCTRL_PLAY */
+		DmaSnd_StartNewFrame();				/* update XSINT + this can clear DMASNDCTRL_PLAY */
 	}
 	else if ((DMASndCtrl_old & DMASNDCTRL_PLAY) && !(nDmaSoundControl & DMASNDCTRL_PLAY))
 	{
 		LOG_TRACE(TRACE_DMASND, "DMA snd control write: stopping dma sound output\n");
+		DmaSnd_Update_XSINT_Line ( MFP_GPIP_STATE_LOW );		/* O/LOW=dma sound idle */
 	}
 }
 
@@ -995,7 +1028,7 @@ void DmaSnd_InterruptHandler_Microwire(void)
 	}
 	
 	/* How many cycle was this sound interrupt delayed (>= 0) */
-	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU8_CYCLE );
+	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU_CYCLE );
 
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
@@ -1317,7 +1350,7 @@ static struct first_order_s *DmaSnd_Bass_Shelf(float g, float fc, float Fs)
 	static struct first_order_s bass;
 	float  a1;
 
-	/* g, fc, Fs must be positve real numbers > 0.0 */
+	/* g, fc, Fs must be positive real numbers > 0.0 */
 	if (g < 1.0)
 		bass.a1 = a1 = (tanf(M_PI*fc/Fs) - g  ) / (tanf(M_PI*fc/Fs) + g  );
 	else
@@ -1338,7 +1371,7 @@ static struct first_order_s *DmaSnd_Treble_Shelf(float g, float fc, float Fs)
 	static struct first_order_s treb;
 	float  a1;
 
-	/* g, fc, Fs must be positve real numbers > 0.0 */
+	/* g, fc, Fs must be positive real numbers > 0.0 */
 	if (g < 1.0)
 		treb.a1 = a1 = (g*tanf(M_PI*fc/Fs) - 1.0) / (g*tanf(M_PI*fc/Fs) + 1.0);
 	else
@@ -1407,3 +1440,34 @@ void DmaSnd_Init_Bass_and_Treble_Tables(void)
 	lmc1992.left_gain = (microwire.leftVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
 	lmc1992.right_gain = (microwire.rightVolume * (Uint32)microwire.masterVolume) * (2.0/(65536.0*65536.0));
 }
+
+
+void DmaSnd_Info(FILE *fp, Uint32 dummy)
+{
+	if (Config_IsMachineST())
+	{
+		fprintf(fp, "ST doesn't include DMA!\n");
+		return;
+	}
+	fprintf(fp, "$FF8900.b : Sound DMA control  : %02x\n", IoMem_ReadByte(0xff8900));
+	fprintf(fp, "$FF8901.b : Sound DMA control  : %02x\n", IoMem_ReadByte(0xff8901));
+	fprintf(fp, "$FF8903.b : Frame Start High   : %02x\n", IoMem_ReadByte(0xff8903));
+	fprintf(fp, "$FF8905.b : Frame Start middle : %02x\n", IoMem_ReadByte(0xff8905));
+	fprintf(fp, "$FF8907.b : Frame Start low    : %02x\n", IoMem_ReadByte(0xff8907));
+	fprintf(fp, "$FF8909.b : Frame Count High   : %02x\n", IoMem_ReadByte(0xff8909));
+	fprintf(fp, "$FF890B.b : Frame Count middle : %02x\n", IoMem_ReadByte(0xff890b));
+	fprintf(fp, "$FF890D.b : Frame Count low    : %02x\n", IoMem_ReadByte(0xff890d));
+	fprintf(fp, "$FF890F.b : Frame End High     : %02x\n", IoMem_ReadByte(0xff890f));
+	fprintf(fp, "$FF8911.b : Frame End middle   : %02x\n", IoMem_ReadByte(0xff8911));
+	fprintf(fp, "$FF8913.b : Frame End low      : %02x\n", IoMem_ReadByte(0xff8913));
+	fprintf(fp, "\n");
+	fprintf(fp, "$FF8920.b : Sound Mode Control : %02x\n", IoMem_ReadByte(0xff8920));
+	fprintf(fp, "$FF8921.b : Sound Mode Control : %02x\n", IoMem_ReadByte(0xff8921));
+	if (Config_IsMachineFalcon())
+	{
+		return;
+	}
+	fprintf(fp, "\n");
+	fprintf(fp, "$FF8922.b : Microwire Data     : %02x\n", IoMem_ReadByte(0xff8922));
+	fprintf(fp, "$FF8922.b : Microwire Mask     : %02x\n", IoMem_ReadByte(0xff8924));
+}
