@@ -6,7 +6,7 @@
 
   Common file access functions.
 */
-const char File_fileid[] = "Hatari file.c : " __DATE__ " " __TIME__;
+const char File_fileid[] = "Hatari file.c";
 
 #include "main.h"
 #include <sys/types.h>
@@ -181,10 +181,95 @@ bool File_DoesFileNameEndWithSlash(char *pszFileName)
 
 
 /*-----------------------------------------------------------------------*/
+/**
+ * Read file via zlib into a newly allocated buffer and return the buffer
+ * or NULL for error. If pFileSize is non-NULL, read file size is set to that.
+ */
+#if HAVE_LIBZ
+Uint8 *File_ZlibRead(const char *pszFileName, long *pFileSize)
+{
+	Uint8 *pFile = NULL;
+	gzFile hGzFile;
+	long nFileSize = 0;
+
+	hGzFile = gzopen(pszFileName, "rb");
+	if (hGzFile != NULL)
+	{
+		/* Find size of file: */
+		do
+		{
+			/* Seek through the file until we hit the end... */
+			char tmp[1024];
+			if (gzread(hGzFile, tmp, sizeof(tmp)) < 0)
+			{
+				gzclose(hGzFile);
+				fprintf(stderr, "Failed to read gzip file!\n");
+					return NULL;
+			}
+		}
+		while (!gzeof(hGzFile));
+		nFileSize = gztell(hGzFile);
+		gzrewind(hGzFile);
+
+		/* Read in... */
+		pFile = malloc(nFileSize);
+		if (pFile)
+			nFileSize = gzread(hGzFile, pFile, nFileSize);
+
+		gzclose(hGzFile);
+	}
+
+	if (pFileSize)
+		*pFileSize = nFileSize;
+
+	return pFile;
+}
+#endif
+
 /**
  * Read file from disk into allocated buffer and return the buffer
- * or NULL for error.  If pFileSize is non-NULL, read file size
- * is set to that.
+ * unmodified, or NULL for error.  If pFileSize is non-NULL, read
+ * file size is set to that.
+ */
+Uint8 *File_ReadAsIs(const char *pszFileName, long *pFileSize)
+{
+	Uint8 *pFile = NULL;
+	long FileSize = 0;
+	FILE *hDiskFile;
+
+	/* Open and read normal file */
+	hDiskFile = fopen(pszFileName, "rb");
+	if (hDiskFile != NULL)
+	{
+		/* Find size of file: */
+		if (fseek(hDiskFile, 0, SEEK_END) == 0)
+		{
+			FileSize = ftell(hDiskFile);
+			if (FileSize > 0 && fseek(hDiskFile, 0, SEEK_SET) == 0)
+			{
+				/* Read in... */
+				pFile = malloc(FileSize);
+				if (pFile)
+					FileSize = fread(pFile, 1, FileSize, hDiskFile);
+			}
+		}
+		fclose(hDiskFile);
+	}
+
+	/* Store size of file we read in (or 0 if failed) */
+	if (pFileSize)
+		*pFileSize = FileSize;
+
+	return pFile;        /* Return to where read in/allocated */
+}
+
+/**
+ * Read file from disk into allocated buffer and return the buffer or
+ * NULL for error.  If file is missing, given additional compression
+ * extension are checked too.  If file has one of the supported
+ * compression extension, its contents are uncompressed (in case of
+ * ZIP, first file in it is read).  If pFileSize is non-NULL, read
+ * file size is set to that.
  */
 Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const ppszExts[])
 {
@@ -205,61 +290,18 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 	/* Is it a gzipped file? */
 	if (File_DoesFileExtensionMatch(filepath, ".gz"))
 	{
-		gzFile hGzFile;
-		/* Open and read gzipped file */
-		hGzFile = gzopen(filepath, "rb");
-		if (hGzFile != NULL)
-		{
-			/* Find size of file: */
-			do
-			{
-				/* Seek through the file until we hit the end... */
-				char tmp[1024];
-				if (gzread(hGzFile, tmp, sizeof(tmp)) < 0)
-				{
-					fprintf(stderr, "Failed to read gzip file!\n");
-					free(filepath);
-					return NULL;
-				}
-			}
-			while (!gzeof(hGzFile));
-			FileSize = gztell(hGzFile);
-			gzrewind(hGzFile);
-			/* Read in... */
-			pFile = malloc(FileSize);
-			if (pFile)
-				FileSize = gzread(hGzFile, pFile, FileSize);
-
-			gzclose(hGzFile);
-		}
+		pFile = File_ZlibRead(filepath, &FileSize);
 	}
 	else if (File_DoesFileExtensionMatch(filepath, ".zip"))
 	{
 		/* It is a .ZIP file! -> Try to load the first file in the archive */
 		pFile = ZIP_ReadFirstFile(filepath, &FileSize, ppszExts);
 	}
-	else          /* It is a normal file */
+	else
 #endif  /* HAVE_LIBZ */
 	{
-		FILE *hDiskFile;
-		/* Open and read normal file */
-		hDiskFile = fopen(filepath, "rb");
-		if (hDiskFile != NULL)
-		{
-			/* Find size of file: */
-			if (fseek(hDiskFile, 0, SEEK_END) == 0)
-			{
-				FileSize = ftell(hDiskFile);
-				if (FileSize > 0 && fseek(hDiskFile, 0, SEEK_SET) == 0)
-				{
-					/* Read in... */
-					pFile = malloc(FileSize);
-					if (pFile)
-						FileSize = fread(pFile, 1, FileSize, hDiskFile);
-				}
-			}
-			fclose(hDiskFile);
-		}
+		/* It is a normal file */
+		pFile = File_ReadAsIs(filepath, &FileSize);
 	}
 	free(filepath);
 
@@ -274,6 +316,7 @@ Uint8 *File_Read(const char *pszFileName, long *pFileSize, const char * const pp
 /*-----------------------------------------------------------------------*/
 /**
  * Save file to disk, return FALSE if errors
+ * If built with ZLib support + file name ends with *.gz, compress it first
  */
 bool File_Save(const char *pszFileName, const Uint8 *pAddress, size_t Size, bool bQueryOverwrite)
 {
@@ -390,6 +433,8 @@ bool File_QueryOverwrite(const char *pszFileName)
 		fmt = "File '%s' exists, overwrite?";
 		/* File does exist, are we OK to overwrite? */
 		szString = malloc(strlen(pszFileName) + strlen(fmt) + 1);
+		if ( !szString )
+			return false;
 		sprintf(szString, fmt, pszFileName);
 		fprintf(stderr, "%s\n", szString);
 		ret = DlgAlert_Query(szString);
@@ -467,7 +512,7 @@ void File_SplitPath(const char *pSrcFileName, char *pDir, char *pName, char *pEx
 	}
 	else
 	{
- 		strcpy(pName, pSrcFileName);
+		strcpy(pName, pSrcFileName);
 		sprintf(pDir, ".%c", PATHSEP);
 	}
 
@@ -510,7 +555,9 @@ char * File_MakePath(const char *pDir, const char *pName, const char *pExt)
 	{
 		filepath[0] = '.';
 		filepath[1] = '\0';
-	} else {
+	}
+	else
+	{
 		strcpy(filepath, pDir);
 	}
 	len = strlen(filepath);
@@ -530,6 +577,35 @@ char * File_MakePath(const char *pDir, const char *pName, const char *pExt)
 	return filepath;
 }
 
+/**
+ * Like File_MakePath(), but puts the string into a pre-allocated buffer
+ * instead of returning a freshly allocated memory buffer.
+ */
+int File_MakePathBuf(char *buf, size_t buflen, const char *pDir,
+                     const char *pName, const char *pExt)
+{
+	char *tmp;
+
+	if (!buflen)
+		return -EINVAL;
+
+	tmp = File_MakePath(pDir, pName, pExt);
+	if (!tmp)
+	{
+		buf[0] = '\0';
+		return -ENOMEM;
+	}
+
+	strncpy(buf, tmp, buflen);
+	free(tmp);
+	if (buf[buflen - 1])
+	{
+		buf[buflen - 1] = '\0';
+		return -E2BIG;
+	}
+
+	return 0;
+}
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -883,13 +959,17 @@ void File_PathShorten(char *path, int dirs)
 	/* ignore last char, it may or may not be '/' */
 	i = strlen(path)-1;
 	assert(i >= 0);
-	while(i > 0 && n < dirs) {
+	while(i > 0 && n < dirs)
+	{
 		if (path[--i] == PATHSEP)
 			n++;
 	}
-	if (path[i] == PATHSEP) {
+	if (path[i] == PATHSEP)
+	{
 		path[i+1] = '\0';
-	} else {
+	}
+	else
+	{
 		path[0] = PATHSEP;
 		path[1] = '\0';
 	}
@@ -918,9 +998,12 @@ void File_HandleDotDirs(char *path)
 	    path[len-1] == '.')
 	{
 		/* go one dir up */
-		if (len == 3) {
+		if (len == 3)
+		{
 			path[1] = 0;		/* already root */
-		} else {
+		}
+		else
+		{
 			char *ptr;
 			path[len-3] = 0;
 			ptr = strrchr(path, PATHSEP);
@@ -946,7 +1029,7 @@ char* WinTmpFile(void)
 
 	/* Gets the temp path env string (no guarantee it's a valid path) */
 	dwRetVal = GetTempPath(MAX_PATH,		/* length of the buffer */
-                           lpTempPathBuffer);		/* buffer for path */
+	                       lpTempPathBuffer);	/* buffer for path */
 	if (dwRetVal > MAX_PATH || (dwRetVal == 0))
 	{
 		Log_Printf(LOG_ERROR, "GetTempPath failed.\n");
@@ -955,9 +1038,9 @@ char* WinTmpFile(void)
 
 	/* Generates a temporary file name */
 	uRetVal = GetTempFileName(lpTempPathBuffer,	/* directory for tmp files */
-                              TEXT("HATARI"),		/* temp file name prefix */
-                              0,			/* create unique name */
-                              szTempFileName);		/* buffer for name */
+	                          TEXT("HATARI"),	/* temp file name prefix */
+	                          0,			/* create unique name */
+	                          szTempFileName);	/* buffer for name */
 	if (uRetVal == 0)
 	{
 		Log_Printf(LOG_ERROR, "GetTempFileName failed.\n");
@@ -966,5 +1049,3 @@ char* WinTmpFile(void)
 	return (char*)szTempFileName;
 }
 #endif
-
-
