@@ -12,7 +12,7 @@
   RAW stream files made with KryoFlux board or CT RAW dumped with an Amiga are also handled
   by the capsimage library.
 */
-const char floppy_ipf_fileid[] = "Hatari floppy_ipf.c : " __DATE__ " " __TIME__;
+const char floppy_ipf_fileid[] = "Hatari floppy_ipf.c";
 
 #include "main.h"
 #include "file.h"
@@ -29,13 +29,12 @@ const char floppy_ipf_fileid[] = "Hatari floppy_ipf.c : " __DATE__ " " __TIME__;
 #include <string.h>
 
 #ifdef HAVE_CAPSIMAGE
-#if CAPSIMAGE_VERSION == 5
-#include <caps5/CapsLibAll.h>
-#else
-#include <caps/fdc.h>
-#define CAPS_LIB_RELEASE	4
-#define CAPS_LIB_REVISION	2
+#ifndef __cdecl
+#define __cdecl  /* CAPS headers need this, but do not define it on their own */
 #endif
+#include <caps/CapsLibAll.h>
+#define CapsLong SDWORD
+#define CapsULong UDWORD
 /* Macro to check release and revision */
 #define	CAPS_LIB_REL_REV	( CAPS_LIB_RELEASE * 100 + CAPS_LIB_REVISION )
 #endif
@@ -97,7 +96,7 @@ static void	IPF_FDC_LogCommand ( Uint8 Command );
 /**
  * Save/Restore snapshot of local variables('MemorySnapShot_Store' handles type)
  * We must take care of whether Hatari was compiled with IPF support of not
- * when saving/restoring snapshots to avoid incompatibilies.
+ * when saving/restoring snapshots to avoid incompatibilities.
  */
 void IPF_MemorySnapShot_Capture(bool bSave)
 {
@@ -227,12 +226,10 @@ bool IPF_FileNameIsIPF(const char *pszFileName, bool bAllowGZ)
 {
 	return ( File_DoesFileExtensionMatch(pszFileName,".ipf" )
 		|| ( bAllowGZ && File_DoesFileExtensionMatch(pszFileName,".ipf.gz") )
-#if CAPS_LIB_REL_REV >= 501
 		|| File_DoesFileExtensionMatch(pszFileName,".raw" )
 		|| ( bAllowGZ && File_DoesFileExtensionMatch(pszFileName,".raw.gz") )
 		|| File_DoesFileExtensionMatch(pszFileName,".ctr" )
 		|| ( bAllowGZ && File_DoesFileExtensionMatch(pszFileName,".ctr.gz") )
-#endif
 		);
 }
 
@@ -411,7 +408,7 @@ void	IPF_Exit ( void )
 
 
 /*
- * Init the ressources to handle the IPF image inserted into a drive (0=A: 1=B:)
+ * Init the resources to handle the IPF image inserted into a drive (0=A: 1=B:)
  */
 bool	IPF_Insert ( int Drive , Uint8 *pImageBuffer , long ImageSize )
 {
@@ -432,7 +429,6 @@ bool	IPF_Insert ( int Drive , Uint8 *pImageBuffer , long ImageSize )
 		return false;
 	}
 
-#if CAPS_LIB_REL_REV >= 501
 	ImageType = CAPSGetImageTypeMemory ( pImageBuffer , ImageSize );
 	if ( ImageType == citError )
 	{
@@ -476,9 +472,6 @@ bool	IPF_Insert ( int Drive , Uint8 *pImageBuffer , long ImageSize )
 		}
 	}
 
-#else
-	ImageType = -1;
-#endif
 
 	if ( CAPSLockImageMemory ( ImageId , pImageBuffer , (CapsULong)ImageSize , DI_LOCK_MEMREF ) == imgeOk )
 	{
@@ -624,7 +617,7 @@ return true;						/* This function is not used for now, always return true */
 
 
 /*
- * When ejecting a disk, free the ressources associated with an IPF image
+ * When ejecting a disk, free the resources associated with an IPF image
  */
 bool	IPF_Eject ( int Drive )
 {
@@ -832,16 +825,13 @@ void	IPF_Drive_Set_DoubleSided ( int Drive , bool value )
 #ifdef HAVE_CAPSIMAGE
 static void	IPF_Drive_Update_Enable_Side ( void )
 {
-#if CAPS_LIB_REL_REV >= 501
 	int	i;
-#endif
 
 	if ( IPF_State.DriveEnabled[ 1 ] )
 	        IPF_State.Fdc.drivemax = MAX_FLOPPYDRIVES;		/* Should be 2 */
 	else
 	        IPF_State.Fdc.drivemax = MAX_FLOPPYDRIVES - 1;		/* Should be 1 */
 
-#if CAPS_LIB_REL_REV >= 501
 	for ( i=0 ; i < MAX_FLOPPYDRIVES ; i++ )
 	{
 		if ( IPF_State.DoubleSided[ i ] )
@@ -849,7 +839,6 @@ static void	IPF_Drive_Update_Enable_Side ( void )
 		else
 			IPF_State.Drive[ i ].diskattr |= CAPSDRIVE_DA_SS;	/* Single sided */
 	}
-#endif
 }
 #endif
 
@@ -910,7 +899,6 @@ void	IPF_FDC_WriteReg ( Uint8 Reg , Uint8 Byte )
 	else
 		LOG_TRACE(TRACE_FDC, "fdc ipf write reg=%d data=0x%x VBL=%d HBL=%d\n" , Reg , Byte , nVBLs , nHBL );
 	
-#if CAPS_LIB_REL_REV >= 501
 	/* In the case of CTR images, we must reset the revolution counter */
 	/* when a command access data on disk and track/side changed since last access */
 	if ( Reg == 0 )
@@ -931,7 +919,6 @@ void	IPF_FDC_WriteReg ( Uint8 Reg , Uint8 Byte )
 			}
 		}
 	}
-#endif
 
 	IPF_Emulate();					/* Update emulation's state up to this point */
 
