@@ -28,7 +28,7 @@
   Also note the 'mirror' (or shadow) registers of the PSG - this is used by most
   games.
 */
-const char IoMem_fileid[] = "Hatari ioMem.c : " __DATE__ " " __TIME__;
+const char IoMem_fileid[] = "Hatari ioMem.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -40,6 +40,7 @@ const char IoMem_fileid[] = "Hatari ioMem.c : " __DATE__ " " __TIME__;
 #include "newcpu.h"
 #include "log.h"
 #include "scc.h"
+#include "fdc.h"
 
 
 static void (*pInterceptReadTable[0x8000])(void);	/* Table with read access handlers */
@@ -214,6 +215,14 @@ static void IoMem_FixAccessForMegaSTE(void)
 		pInterceptReadTable[addr - 0xff8000] = SCC_IoMem_ReadByte;
 		pInterceptWriteTable[addr - 0xff8000] = SCC_IoMem_WriteByte;
 	}
+
+	/* The Mega-STE can choose between DD and HD mode when reading floppy */
+	/* This uses word register at 0xff860e */
+	for (addr = 0xff860e; addr <= 0xff860f; addr++)
+	{
+		pInterceptReadTable[addr - 0xff8000] = FDC_DensityMode_ReadWord;
+		pInterceptWriteTable[addr - 0xff8000] = FDC_DensityMode_WriteWord;
+	}
 }
 
 
@@ -267,26 +276,21 @@ void IoMem_Init(void)
 	switch (ConfigureParams.System.nMachineType)
 	{
 	 case MACHINE_ST:
-		IoMem_FixVoidAccessForST();
 		pInterceptAccessFuncs = IoMemTable_ST;
 		break;
 	 case MACHINE_MEGA_ST:
-		IoMem_FixVoidAccessForMegaST();
 		pInterceptAccessFuncs = IoMemTable_ST;
 		break;
 	 case MACHINE_STE:
 		pInterceptAccessFuncs = IoMemTable_STE;
 		break;
 	 case MACHINE_MEGA_STE:
-		IoMem_FixAccessForMegaSTE();
 		pInterceptAccessFuncs = IoMemTable_STE;
 		break;
 	 case MACHINE_TT:
 		pInterceptAccessFuncs = IoMemTable_TT;
 		break;
 	 case MACHINE_FALCON:
-		if (falconBusMode == STE_BUS_COMPATIBLE)
-			IoMem_FixVoidAccessForCompatibleFalcon();
 		pInterceptAccessFuncs = IoMemTable_Falcon;
 		break;
 	 default:
@@ -315,9 +319,23 @@ void IoMem_Init(void)
 		}
 	}
 
-	/* Set registers for Falcon DSP emulation */
+	/* After the IO access handlers were set, some machines with common IoMemTable_xxx */
+	/* will require some extra changes (eg: ST vs MegaST, STE ve MegaSTE) */
+	if ( ConfigureParams.System.nMachineType == MACHINE_ST )
+		IoMem_FixVoidAccessForST();
+	else if ( ConfigureParams.System.nMachineType == MACHINE_MEGA_ST )
+		IoMem_FixVoidAccessForMegaST();
+	else if ( ConfigureParams.System.nMachineType == MACHINE_MEGA_STE )
+		IoMem_FixAccessForMegaSTE();
+
+
+	/* Set registers for Falcon */
 	if (Config_IsMachineFalcon())
 	{
+		if (falconBusMode == STE_BUS_COMPATIBLE)
+			IoMem_FixVoidAccessForCompatibleFalcon();
+
+		/* Set registers for Falcon DSP emulation */
 		switch (ConfigureParams.System.nDSPType)
 		{
 #if ENABLE_DSP_EMU
@@ -433,10 +451,10 @@ uae_u32 REGPARAM3 IoMem_bget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr < 0xff8000 || !regs.s)
+	if (addr < 0xff8000 || !is_super_access(true))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -450,7 +468,7 @@ uae_u32 REGPARAM3 IoMem_bget(uaecptr addr)
 	/* Check if we read from a bus-error region */
 	if (nBusErrorAccesses == 1)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -488,10 +506,10 @@ uae_u32 REGPARAM3 IoMem_wget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr < 0xff8000 || !regs.s)
+	if (addr < 0xff8000 || !is_super_access(true))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 	if (addr > 0xfffffe)
@@ -517,7 +535,7 @@ uae_u32 REGPARAM3 IoMem_wget(uaecptr addr)
 	/* Check if we completely read from a bus-error region */
 	if (nBusErrorAccesses == 2)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -555,10 +573,10 @@ uae_u32 REGPARAM3 IoMem_lget(uaecptr addr)
 
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	if (addr < 0xff8000 || !regs.s)
+	if (addr < 0xff8000 || !is_super_access(true))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 	if (addr > 0xfffffc)
@@ -587,7 +605,7 @@ uae_u32 REGPARAM3 IoMem_lget(uaecptr addr)
 	/* Check if we completely read from a bus-error region */
 	if (nBusErrorAccesses == 4)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, 0);
 		return -1;
 	}
 
@@ -623,10 +641,10 @@ void REGPARAM3 IoMem_bput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IOMEM_WR, "IO write.b $%08x = $%02x pc=%x\n", IoAccessFullAddress, val&0xff, M68000_GetPC());
 
-	if (addr < 0xff8000 || !regs.s)
+	if (addr < 0xff8000 || !is_super_access(false))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 
@@ -642,7 +660,7 @@ void REGPARAM3 IoMem_bput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 1)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, val);
 	}
 }
 
@@ -674,10 +692,10 @@ void REGPARAM3 IoMem_wput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IOMEM_WR, "IO write.w $%08x = $%04x pc=%x\n", IoAccessFullAddress, val&0xffff, M68000_GetPC());
 
-	if (addr < 0x00ff8000 || !regs.s)
+	if (addr < 0x00ff8000 || !is_super_access(false))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 	if (addr > 0xfffffe)
@@ -705,7 +723,7 @@ void REGPARAM3 IoMem_wput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 2)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA, val);
 	}
 }
 
@@ -737,10 +755,10 @@ void REGPARAM3 IoMem_lput(uaecptr addr, uae_u32 val)
 
 	LOG_TRACE(TRACE_IOMEM_WR, "IO write.l $%08x = $%08x pc=%x\n", IoAccessFullAddress, val, M68000_GetPC());
 
-	if (addr < 0xff8000 || !regs.s)
+	if (addr < 0xff8000 || !is_super_access(false))
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, val);
 		return;
 	}
 	if (addr > 0xfffffc)
@@ -771,11 +789,32 @@ void REGPARAM3 IoMem_lput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 4)
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA, val);
 	}
 }
 
 
+/*-------------------------------------------------------------------------*/
+/**
+ * Check if an address inside the IO mem region would return a bus error in case of a read/write access
+ * We only check if it would give a bus error on read access, as in our case it would give
+ * a bus error too in case of a write
+ */
+bool	IoMem_CheckBusError ( Uint32 addr )
+{
+	addr &= 0xffff;
+
+	if ( addr < 0x8000 )
+		return true;
+
+	if ( ( pInterceptReadTable[ addr - 0x8000 ] == IoMem_BusErrorOddReadAccess )
+	  || ( pInterceptReadTable[ addr - 0x8000 ] == IoMem_BusErrorEvenReadAccess ) )
+		return true;
+
+	return false;
+}
+
+
 /*-------------------------------------------------------------------------*/
 /**
  * This handler will be called if a ST program tries to read from an address
