@@ -6,7 +6,7 @@
 
   Table with hardware IO handlers for the Falcon.
 */
-const char IoMemTabFalc_fileid[] = "Hatari ioMemTabFalcon.c : " __DATE__ " " __TIME__;
+const char IoMemTabFalc_fileid[] = "Hatari ioMemTabFalcon.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -178,9 +178,9 @@ static void IoMemTabFalcon_BusCtrl_ReadByte(void)
  * Logic is inverted, i.e. connected means the corresponding bit is 0.
  * Switch 8 is represented by the highest bit in the register.
  */
-static void IoMemTabFalc_Switches_ReadByte(void)
+Uint8 IoMemTabFalcon_DIPSwitches_Read(void)
 {
-	IoMem_WriteByte(0xff9200, 0xbf);
+	return 0xbf;
 }
 
 
@@ -192,8 +192,7 @@ static void IoMemTabFalc_Compatible_ReadByte(void)
 {
 	if (nIoMemAccessSize != SIZE_BYTE || IoMem_IsFalconBusMode())
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
-		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, nIoMemAccessSize, BUS_ERROR_ACCESS_DATA, 0);
 	}
 }
 
@@ -201,8 +200,7 @@ static void IoMemTabFalc_Compatible_WriteByte(void)
 {
 	if (nIoMemAccessSize != SIZE_BYTE || IoMem_IsFalconBusMode())
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE,
-		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, nIoMemAccessSize, BUS_ERROR_ACCESS_DATA, 0);
 	}
 }
 
@@ -214,8 +212,7 @@ static void IoMemTabFalc_Compatible_ReadWord(void)
 {
 	if (nIoMemAccessSize == SIZE_BYTE || IoMem_IsFalconBusMode())
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
-		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, nIoMemAccessSize, BUS_ERROR_ACCESS_DATA, 0);
 	}
 }
 
@@ -223,8 +220,7 @@ static void IoMemTabFalc_Compatible_WriteWord(void)
 {
 	if (nIoMemAccessSize == SIZE_BYTE || IoMem_IsFalconBusMode())
 	{
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE,
-		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, nIoMemAccessSize, BUS_ERROR_ACCESS_DATA,0);
 	}
 }
 
@@ -315,8 +311,7 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_Falcon[] =
 	{ 0xff860b, SIZE_BYTE, FDC_DmaAddress_ReadByte, FDC_DmaAddress_WriteByte },		/* DMA base and counter med byte  */
 	{ 0xff860c, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
 	{ 0xff860d, SIZE_BYTE, FDC_DmaAddress_ReadByte, FDC_DmaAddress_WriteByte },		/* DMA base and counter low byte  */
-	{ 0xff860e, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff860f, SIZE_BYTE, FDC_FloppyMode_ReadByte, FDC_FloppyMode_WriteByte },             /* Floppy mode (?) register */
+	{ 0xff860e, SIZE_WORD, FDC_DensityMode_ReadWord , FDC_DensityMode_WriteWord },		/* Choose DD/HD mode */
 
 	{ 0xff8800, SIZE_BYTE, PSG_ff8800_ReadByte, PSG_ff8800_WriteByte },
 	{ 0xff8801, SIZE_BYTE, PSG_ff880x_ReadByte, PSG_ff8801_WriteByte },
@@ -402,8 +397,7 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_Falcon[] =
 
 	{ 0xff8c80, 8, SCC_IoMem_ReadByte, SCC_IoMem_WriteByte },                         /* SCC */
 
-	{ 0xff9200, SIZE_BYTE, IoMemTabFalc_Switches_ReadByte, IoMem_WriteWithoutInterception }, /* Falcon switches */
-	{ 0xff9201, SIZE_BYTE, Joy_StePadButtons_ReadByte, IoMem_WriteWithoutInterception }, /* Joypad fire buttons */
+	{ 0xff9200, SIZE_WORD, Joy_StePadButtons_DIPSwitches_ReadWord, Joy_StePadButtons_DIPSwitches_WriteWord },    /* Joypad fire buttons + Falcon DIP Switches */
 	{ 0xff9202, SIZE_WORD, Joy_StePadMulti_ReadWord, Joy_StePadMulti_WriteWord },     /* Joypad directions/buttons/selection */
 	{ 0xff9210, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                         /* No bus error here */
 	{ 0xff9211, SIZE_BYTE, IoMem_VoidRead, IoMem_WriteWithoutInterception },          /* Joypad 0 X position (?) */
