@@ -8,7 +8,7 @@
 
   NOTE: The ST uses the joystick port 1 as the default controller.
 */
-const char Joy_fileid[] = "Hatari joy.c : " __DATE__ " " __TIME__;
+const char Joy_fileid[] = "Hatari joy.c";
 
 #include <SDL.h>
 
@@ -31,6 +31,7 @@ const char Joy_fileid[] = "Hatari joy.c : " __DATE__ " " __TIME__;
 
 #define JOYREADING_BUTTON1  1		/* bit 0 */
 #define JOYREADING_BUTTON2  2		/* bit 1 */
+#define JOYREADING_BUTTON3  4		/* bit 2 */
 
 typedef struct
 {
@@ -200,14 +201,28 @@ void Joy_UnInit(void)
  */
 static bool Joy_ReadJoystick(int nSdlJoyID, JOYREADING *pJoyReading)
 {
+	unsigned hat = SDL_JoystickGetHat(sdlJoystick[nSdlJoyID], 0);
+
 	/* Joystick is OK, read position from the configured joystick axis */
 	pJoyReading->XPos = SDL_JoystickGetAxis(sdlJoystick[nSdlJoyID], pJoyReading->XAxisID);
 	pJoyReading->YPos = SDL_JoystickGetAxis(sdlJoystick[nSdlJoyID], pJoyReading->YAxisID);
+	/* Similarly to other emulators that support hats, override axis readings with hats */
+	if (hat & SDL_HAT_LEFT)
+		pJoyReading->XPos = -32768;
+	if (hat & SDL_HAT_RIGHT)
+		pJoyReading->XPos = 32767;
+	if (hat & SDL_HAT_UP)
+		pJoyReading->YPos = -32768;
+	if (hat & SDL_HAT_DOWN)
+		pJoyReading->YPos = 32767;
 	/* Sets bit #0 if button #1 is pressed: */
 	pJoyReading->Buttons = SDL_JoystickGetButton(sdlJoystick[nSdlJoyID], 0);
 	/* Sets bit #1 if button #2 is pressed: */
 	if (SDL_JoystickGetButton(sdlJoystick[nSdlJoyID], 1))
 		pJoyReading->Buttons |= JOYREADING_BUTTON2;
+	/* Sets bit #2 if button #3 is pressed: */
+	if (SDL_JoystickGetButton(sdlJoystick[nSdlJoyID], 2))
+		pJoyReading->Buttons |= JOYREADING_BUTTON3;
 
 	return true;
 }
@@ -281,7 +296,7 @@ Uint8 Joy_GetStickData(int nStJoyId)
 		if (JoyReading.Buttons & JOYREADING_BUTTON1)
 			nData |= ATARIJOY_BITMASK_FIRE;
 
-		/* Enable PC Joystick button 2 to mimick space bar (For XenonII, Flying Shark etc...) */
+		/* Enable PC Joystick button 2 to mimic space bar (For XenonII, Flying Shark etc...) */
 		if (nStJoyId == JOYID_JOYSTICK1 && (JoyReading.Buttons & JOYREADING_BUTTON2))
 		{
 			if (ConfigureParams.Joysticks.Joy[nStJoyId].bEnableJumpOnFire2)
@@ -298,6 +313,14 @@ Uint8 Joy_GetStickData(int nStJoyId)
 				}
 			}
 		}
+
+		/* PC Joystick button 3 is autofire button for ST joystick button */
+		if (JoyReading.Buttons & JOYREADING_BUTTON3)
+		{
+			nData |= ATARIJOY_BITMASK_FIRE;
+			if ((nVBLs&0x7)<4)
+				nData &= ~ATARIJOY_BITMASK_FIRE;          /* Remove top bit! */
+		}
 	}
 
 	/* Ignore fire button every 8 frames if enabled autofire (for both cursor emulation and joystick) */
@@ -519,11 +542,24 @@ bool Joy_KeyUp(int symkey, int modkey)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Read from STE joypad buttons register (0xff9201)
+ * Read from STE / Falcon joypad buttons register (0xff9200)
+ * - On MegaSTE and Falcon, the byte at $ff9200 also contains the state of the 8 DIP switches
+ *   available on the motherboard
+ * - Note that on STE/MegaSTE $ff9200 can only be accessed as word, not byte. $ff9201 can be
+ *   accessed as byte
  */
-void Joy_StePadButtons_ReadByte(void)
+void Joy_StePadButtons_DIPSwitches_ReadWord(void)
 {
-	Uint8 nData = 0xff;
+	Uint16 nData = 0xff;
+	Uint8 DIP;
+
+	if ( !Config_IsMachineFalcon()
+	  && ( nIoMemAccessSize == SIZE_BYTE ) && ( IoAccessCurrentAddress == 0xff9200 ) )
+	{
+		/* This register does not like to be accessed in byte mode at $ff9200 */
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
+		return;
+	}
 
 	if (ConfigureParams.Joysticks.Joy[JOYID_STEPADA].nJoystickMode != JOYSTICK_DISABLED
 	    && (nSteJoySelect & 0x0f) != 0x0f)
@@ -581,8 +617,35 @@ void Joy_StePadButtons_ReadByte(void)
 		}
 	}
 
-	Dprintf(("0xff9201 -> 0x%04x\n", nData));
-	IoMem_WriteByte(0xff9201, nData);
+	/* On MegaSTE and Falcon, add the state of the 8 DIP Switches in upper byte */
+	if ( Config_IsMachineMegaSTE() )
+		DIP = IoMemTabMegaSTE_DIPSwitches_Read();
+	else if ( Config_IsMachineFalcon() )
+		DIP = IoMemTabFalcon_DIPSwitches_Read();
+	else
+		DIP = 0xff;				/* STE, No DIP switches */
+	nData |= ( DIP << 8 );
+
+	Dprintf(("0xff9200 -> 0x%04x\n", nData));
+	IoMem_WriteWord(0xff9200, nData);
+}
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Write to STE / Falcon joypad buttons register (0xff9200)
+ * This does nothing, but we still check that $ff9200 is not accessed as byte
+ * on STE/MegaSTE, else we trigger a bus error
+ */
+void Joy_StePadButtons_DIPSwitches_WriteWord(void)
+{
+	if ( !Config_IsMachineFalcon()
+	  && ( nIoMemAccessSize == SIZE_BYTE ) && ( IoAccessCurrentAddress == 0xff9200 ) )
+	{
+		/* This register does not like to be accessed in byte mode at $ff9200 */
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
+		return;
+	}
 }
 
 
@@ -670,8 +733,7 @@ void Joy_SteLightpenX_ReadWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
-		                BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
 		return;
 	}
 
@@ -690,8 +752,7 @@ void Joy_SteLightpenY_ReadWord(void)
 	if (nIoMemAccessSize == SIZE_BYTE)
 	{
 		/* This register does not like to be accessed in byte mode */
-		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
-		                BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA, 0);
 		return;
 	}
 
