@@ -6,7 +6,7 @@
 
   Here we process a key press and the remapping of the scancodes.
 */
-const char Keymap_fileid[] = "Hatari keymap.c : " __DATE__ " " __TIME__;
+const char Keymap_fileid[] = "Hatari keymap.c";
 
 #include <ctype.h>
 #include "main.h"
@@ -346,6 +346,15 @@ static char Keymap_PcToStScanCode(SDL_keysym* pKeySym)
 	 case SDL_SCANCODE_RCTRL: return 0x1d;
 	 case SDL_SCANCODE_RSHIFT: return 0x36;
 	 default:
+		if (!pKeySym->scancode && pKeySym->sym)
+		{
+			/* assume SimulateKey
+			 * -> KeyUp/Down
+			 *    -> Remap (with scancode mode configured)
+			 *       -> PcToStScanCode
+			 */
+			return Keymap_SymbolicToStScanCode(pKeySym);
+		}
 		Log_Printf(LOG_WARN, "Unhandled scancode 0x%x!\n", pKeySym->scancode);
 		return -1;
 	}
@@ -513,7 +522,7 @@ static char Keymap_PcToStScanCode(SDL_keysym* keysym)
 			        SDL_GetKeyName(keysym->sym), keysym->scancode, keysym->sym);
 			return (keysym->scancode - 8);
 		}
-	 	break;
+		break;
 	}
 }
 
@@ -531,7 +540,6 @@ static char Keymap_GetKeyPadScanCode(SDL_keysym* pKeySym)
 	{
 		switch (pKeySym->sym)
 		{
-		 case SDLK_KP0:  return 0x70;  /* NumPad 0 */
 		 case SDLK_KP1:  return 0x6d;  /* NumPad 1 */
 		 case SDLK_KP2:  return 0x6e;  /* NumPad 2 */
 		 case SDLK_KP3:  return 0x6f;  /* NumPad 3 */
@@ -548,7 +556,6 @@ static char Keymap_GetKeyPadScanCode(SDL_keysym* pKeySym)
 	{
 		switch (pKeySym->sym)
 		{
-		 case SDLK_KP0:  return 0x70;  /* NumPad 0 */
 		 case SDLK_KP1:  return 0x6d;  /* NumPad 1 */
 		 case SDLK_KP2:  return 0x50;  /* Cursor down */
 		 case SDLK_KP3:  return 0x6f;  /* NumPad 3 */
@@ -574,7 +581,7 @@ static char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 	/* Check for keypad first so we can handle numlock */
 	if (ConfigureParams.Keyboard.nKeymapType != KEYMAP_LOADED)
 	{
-		if (pKeySym->sym >= SDLK_KP0 && pKeySym->sym <= SDLK_KP9)
+		if (pKeySym->sym >= SDLK_KP1 && pKeySym->sym <= SDLK_KP9)
 		{
 			return Keymap_GetKeyPadScanCode(pKeySym);
 		}
@@ -618,7 +625,7 @@ void Keymap_LoadRemapFile(char *pszFileName)
 
 	if (!*pszFileName)
 		return;
-	
+
 	/* Attempt to load file */
 	if (!File_Exists(pszFileName))
 	{
@@ -632,7 +639,7 @@ void Keymap_LoadRemapFile(char *pszFileName)
 			   " open keymap file '%s'\n", pszFileName);
 		return;
 	}
-	
+
 	while (!feof(in) && idx < KBD_MAX_SCANCODE)
 	{
 		/* Read line from file */
