@@ -72,7 +72,7 @@
 /*			with some tables/heuristics anymore.						*/
 
 
-const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
+const char M68000_fileid[] = "Hatari m68000.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -251,7 +251,7 @@ void M68000_Reset(bool bCold)
 	}
 	/* Now reset the UAE CPU core */
 	m68k_reset();
-	Cart_PatchCpuTables();
+	M68000_PatchCpuTables();
 #endif
 
 	BusMode = BUS_MODE_CPU;
@@ -423,7 +423,45 @@ void M68000_CheckCpuSettings(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
+/**
+ * Patch the cpu tables to intercept some opcodes used for Gemdos HD
+ * emulation, extended VDI more or for NatFeats.
+ */
+void M68000_PatchCpuTables(void)
+{
+	if (Cart_UseBuiltinCartridge())
+	{
+		/* Hatari's specific illegal opcodes */
+		cpufunctbl[GEMDOS_OPCODE] = OpCode_GemDos;	/* 0x0008 */
+		cpufunctbl[PEXEC_OPCODE] = OpCode_Pexec;	/* 0x0009 */
+		cpufunctbl[SYSINIT_OPCODE] = OpCode_SysInit;	/* 0x000a */
+		cpufunctbl[VDI_OPCODE] = OpCode_VDI;		/* 0x000c */
+	}
+	else
+	{
+		/* No built-in cartridge loaded : set same handler as 0x4afc (illegal) */
+		cpufunctbl[GEMDOS_OPCODE] = cpufunctbl[0x4afc];   /* 0x0008 */
+		cpufunctbl[PEXEC_OPCODE] = cpufunctbl[0x4afc];    /* 0x0009*/
+		cpufunctbl[SYSINIT_OPCODE] = cpufunctbl[0x4afc];  /* 0x000a */
+		cpufunctbl[VDI_OPCODE] = cpufunctbl[0x4afc];      /* 0x000c */
+	}
+
+	/* Install opcodes for Native Features? */
+	if (ConfigureParams.Log.bNatFeats)
+	{
+		/* illegal opcodes for emulators Native Features */
+		cpufunctbl[NATFEAT_ID_OPCODE] = OpCode_NatFeat_ID;	/* 0x7300 */
+		cpufunctbl[NATFEAT_CALL_OPCODE] = OpCode_NatFeat_Call;	/* 0x7301 */
+	}
+	else
+	{
+		/* No Native Features : set same handler as 0x4afc (illegal) */
+		cpufunctbl[NATFEAT_ID_OPCODE] = cpufunctbl[ 0x4afc ];	/* 0x7300 */
+		cpufunctbl[NATFEAT_CALL_OPCODE] = cpufunctbl[ 0x4afc ];	/* 0x7300 */
+	}
+}
+
+
 /**
  * Save/Restore snapshot of CPU variables ('MemorySnapShot_Store' handles type)
  */
@@ -519,7 +557,7 @@ void M68000_MemorySnapShot_Capture(bool bSave)
 
 	if (!bSave)
 	{
-		Cart_PatchCpuTables();
+		M68000_PatchCpuTables();
 
 		M68000_SetPC(regs.pc);
 		/* MakeFromSR() must not swap stack pointer */
@@ -599,9 +637,12 @@ bool M68000_IsVerboseBusError(uint32_t pc, uint32_t addr)
 
 /**
  * BUSERROR - Access outside valid memory range.
- * Use bRead = 0 for write errors and bRead = 1 for read errors!
+ *   ReadWrite : BUS_ERROR_READ in case of a read or BUS_ERROR_WRITE in case of write
+ *   Size : BUS_ERROR_SIZE_BYTE or BUS_ERROR_SIZE_WORD or BUS_ERROR_SIZE_LONG
+ *   AccessType : BUS_ERROR_ACCESS_INSTR or BUS_ERROR_ACCESS_DATA
+ *   val : value we wanted to write in case of a BUS_ERROR_WRITE
  */
-void M68000_BusError ( Uint32 addr , int ReadWrite , int Size , int AccessType )
+void M68000_BusError ( Uint32 addr , int ReadWrite , int Size , int AccessType , uae_u32 val )
 {
 	LOG_TRACE(TRACE_CPU_EXCEPTION, "Bus error %s at address $%x PC=$%x.\n",
 	          ReadWrite ? "reading" : "writing", addr, M68000_InstrPC);
@@ -613,11 +654,25 @@ void M68000_BusError ( Uint32 addr , int ReadWrite , int Size , int AccessType )
 		bBusErrorReadWrite = ReadWrite;
 		M68000_SetSpecial(SPCFLAG_BUSERROR);		/* The exception will be done in newcpu.c */
 	}
+#else
+#define WINUAE_HANDLE_BUS_ERROR
+#ifdef WINUAE_HANDLE_BUS_ERROR
+
+	bool	read , ins;
+	int	size;
+
+	if ( ReadWrite == BUS_ERROR_READ )		read = true; else read = false;
+	if ( AccessType == BUS_ERROR_ACCESS_INSTR )	ins = true; else ins = false;
+	if ( Size == BUS_ERROR_SIZE_BYTE )		size = sz_byte;
+	else if ( Size == BUS_ERROR_SIZE_WORD )		size = sz_word;
+	else						size = sz_long;
+	hardware_exception2 ( addr , val , read , ins , size );
 #else
 	/* With WinUAE's cpu, on a bus error instruction will be correctly aborted before completing, */
 	/* so we don't need to check if the opcode already generated a bus error or not */
 	exception2 ( addr , ReadWrite , Size , AccessType );
 #endif
+#endif
 }
 
 
