@@ -6,7 +6,7 @@
 
   Main initialization and event handling routines.
 */
-const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
+const char Main_fileid[] = "Hatari main.c";
 
 #include <time.h>
 #include <errno.h>
@@ -20,10 +20,12 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "dialog.h"
 #include "audio.h"
 #include "joy.h"
+#include "file.h"
 #include "floppy.h"
 #include "floppy_ipf.h"
 #include "floppy_stx.h"
 #include "gemdos.h"
+#include "mfp.h"
 #include "fdc.h"
 #include "hdc.h"
 #include "ide.h"
@@ -35,6 +37,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "midi.h"
+#include "ncr5380.h"
 #include "nvram.h"
 #include "paths.h"
 #include "printer.h"
@@ -79,7 +82,10 @@ static int nVBLSlowdown = 1;		  /* host VBL wait multiplier */
 
 static bool bEmulationActive = true;      /* Run emulation when started */
 static bool bAccurateDelays;              /* Host system has an accurate SDL_Delay()? */
+
 static bool bIgnoreNextMouseMotion = false;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
+static bool bAllowMouseWarp = true;       /* disabled when Hatari window loses mouse pointer / key focus */
+
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -355,7 +361,7 @@ void Main_WaitOnVbl(void)
 		if (nFrameSkips < ConfigureParams.Screen.nFrameSkips)
 		{
 			nFrameSkips += 1;
-			// Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
+			Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
 		}
 		/* Only update DestTicks for next VBL */
 		DestTicks = CurrentTicks + FrameDuration_micro;
@@ -370,7 +376,7 @@ void Main_WaitOnVbl(void)
 	    && 2*nDelay > FrameDuration_micro/nFrameSkips)
 	{
 		nFrameSkips -= 1;
-		// Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
+		Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
 	}
 
 	if (bAccurateDelays)
@@ -434,12 +440,17 @@ static void Main_CheckForAccurateDelays(void)
  * Set mouse pointer to new x,y coordinates and set flag to ignore
  * the mouse event that is generated by SDL_WarpMouse().
  *
- * Skip the request is it's not position restore and mouse warping is disabled.
+ * Skip the request if:
+ * - it's not position restore and mouse warping is disabled, or
+ * - mouse warp disable due to mouse leaving Hatari window or focus loss
+ *   (i.e. user isn't interacting with it)
  */
 void Main_WarpMouse(int x, int y, bool restore)
 {
 	if (!(restore || ConfigureParams.Screen.bMouseWarp))
 		return;
+	if (!bAllowMouseWarp)
+		return;
 #if WITH_SDL2
 	SDL_WarpMouseInWindow(sdlWindow, x, y);
 #else
@@ -634,27 +645,52 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_WINDOWEVENT:
-			if (event.window.event == SDL_WINDOWEVENT_EXPOSED
-			    && !ConfigureParams.Screen.bUseSdlRenderer)
-			{
-				/* Hack: Redraw screen here when going into
-				 * fullscreen mode without SDL renderer */
-				sdlscrn = SDL_GetWindowSurface(sdlWindow);
-				Screen_SetFullUpdate();
-				Statusbar_Init(sdlscrn);
-			}
-			/* Note: any changes here should most likely be done
-			 * also in sdlgui.c::SDLGui_DoDialog()
-			 */
-			if (event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED
-			    || event.window.event == SDL_WINDOWEVENT_RESTORED
-			    || event.window.event == SDL_WINDOWEVENT_EXPOSED)
-			{
+			Log_Printf(LOG_DEBUG, "SDL2 window event: 0x%x\n", event.window.event);
+			switch(event.window.event) {
+			case SDL_WINDOWEVENT_EXPOSED:
+				if (!ConfigureParams.Screen.bUseSdlRenderer)
+				{
+					/* Hack: Redraw screen here when going into
+					 * fullscreen mode without SDL renderer */
+					sdlscrn = SDL_GetWindowSurface(sdlWindow);
+					Screen_SetFullUpdate();
+					Statusbar_Init(sdlscrn);
+				}
+				/* fall through */
+			case SDL_WINDOWEVENT_RESTORED:
+				/* Note: any changes here should most likely
+				 * be done also in sdlgui.c::SDLGui_DoDialog()
+				 */
 				SDL_UpdateRect(sdlscrn, 0, 0, 0, 0);
+				break;
+			case SDL_WINDOWEVENT_SIZE_CHANGED:
+				/* internal & external window size changes */
+				Screen_SetTextureScale(sdlscrn->w, sdlscrn->h,
+						       event.window.data1,
+						       event.window.data2, false);
+				SDL_UpdateRect(sdlscrn, 0, 0, 0, 0);
+				break;
+				/* mouse & keyboard focus */
+			case SDL_WINDOWEVENT_ENTER:
+			case SDL_WINDOWEVENT_FOCUS_GAINED:
+				bAllowMouseWarp = true;
+				break;
+			case SDL_WINDOWEVENT_LEAVE:
+			case SDL_WINDOWEVENT_FOCUS_LOST:
+				bAllowMouseWarp = false;
+				break;
 			}
+#else /* !WITH_SDL2 */
+		case SDL_ACTIVEEVENT:
+			Log_Printf(LOG_DEBUG, "SDL1 active event 0x%x = %d\n",
+				   event.active.state, event.active.gain);
+			if (event.active.gain)
+				bAllowMouseWarp = true;
+			else
+				bAllowMouseWarp = false;
+#endif /* !WITH_SDL2 */
 			bContinueProcessing = true;
 			break;
-#endif /* WITH_SDL2 */
 
 		 default:
 			/* don't let unknown events delay event processing */
@@ -733,6 +769,7 @@ static void Main_Init(void)
 	Resolution_Init();
 	SDLGui_Init();
 	Printer_Init();
+	MFP_Init(MFP_Array);
 	RS232_Init();
 	SCC_Init();
 	Midi_Init();
@@ -754,11 +791,12 @@ static void Main_Init(void)
 
 	/* Init HD emulation */
 	HDC_Init();
+	Ncr5380_Init();
 	Ide_Init();
 	GemDOS_Init();
 	if (ConfigureParams.HardDisk.bUseHardDiskDirectories)
 	{
-		/* uses variables set by HDC_Init()! */
+		/* uses variables set by HDC_Init/Ncr5380_Init/Ide_Init */
 		GemDOS_InitDrives();
 	}
 
@@ -794,6 +832,7 @@ static void Main_UnInit(void)
 	Screen_ReturnFromFullScreen();
 	Floppy_UnInit();
 	HDC_UnInit();
+	Ncr5380_UnInit();
 	Midi_UnInit();
 	SCC_UnInit();
 	RS232_UnInit();
@@ -825,20 +864,19 @@ static void Main_UnInit(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Load initial configuration file(s)
+ * Load initial configuration files. The global config file is skipped in
+ * test mode (i.e. if the HATARI_TEST environment variable has been set),
+ * so that the test has always full control over the configuration settings.
  */
 static void Main_LoadInitialConfig(void)
 {
 	char *psGlobalConfig;
 
 	psGlobalConfig = malloc(FILENAME_MAX);
-	if (psGlobalConfig)
+	if (psGlobalConfig && !getenv("HATARI_TEST"))
 	{
-#if defined(__AMIGAOS4__)
-		strncpy(psGlobalConfig, CONFDIR"hatari.cfg", FILENAME_MAX);
-#else
-		snprintf(psGlobalConfig, FILENAME_MAX, CONFDIR"%chatari.cfg", PATHSEP);
-#endif
+		File_MakePathBuf(psGlobalConfig, FILENAME_MAX, CONFDIR,
+		                 "hatari", "cfg");
 		/* Try to load the global configuration file */
 		Configuration_Load(psGlobalConfig);
 
@@ -855,24 +893,47 @@ static void Main_LoadInitialConfig(void)
  */
 static void Main_StatusbarSetup(void)
 {
-	const char *name = NULL;
+	struct {
+		const int id;
+		bool mod;
+		char *name;
+	} keys[] = {
+		{ SHORTCUT_OPTIONS, false, NULL },
+		{ SHORTCUT_MOUSEGRAB, false, NULL }
+	};
+	const char *name;
+	bool named;
 	SDLKey key;
+	int i;
 
-	key = ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS];
-	if (!key)
-		key = ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS];
-	if (key)
-		name = SDL_GetKeyName(key);
-	if (name)
+	named = false;
+	for (i = 0; i < ARRAY_SIZE(keys); i++)
 	{
-		char message[24], *keyname;
-#ifdef _MUDFLAP
-		__mf_register((void*)name, 32, __MF_TYPE_GUESS, "SDL keyname");
-#endif
-		keyname = Str_ToUpper(strdup(name));
-		snprintf(message, sizeof(message), "Press %s for Options", keyname);
-		free(keyname);
-
+		key = ConfigureParams.Shortcut.withoutModifier[keys[i].id];
+		if (!key)
+		{
+			key = ConfigureParams.Shortcut.withModifier[keys[i].id];
+			if (!key)
+				continue;
+			keys[i].mod = true;
+		}
+		name = SDL_GetKeyName(key);
+		if (!name)
+			continue;
+		keys[i].name = Str_ToUpper(strdup(name));
+		named = true;
+	}
+	if (named)
+	{
+		char message[60];
+		snprintf(message, sizeof(message), "Press %s%s for Options, %s%s for mouse grab toggle",
+			 keys[0].mod ? "AltGr+": "", keys[0].name,
+			 keys[1].mod ? "AltGr+": "", keys[1].name);
+		for (i = 0; i < ARRAY_SIZE(keys); i++)
+		{
+			if (keys[i].name)
+				free(keys[i].name);
+		}
 		Statusbar_AddMessage(message, 5000);
 	}
 	/* update information loaded by Main_Init() */
