@@ -11,7 +11,7 @@
   - Add the option information to HatariOptions[]
   - Add required actions for that ID to switch in Opt_ParseParameters()
 */
-const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
+const char Options_fileid[] = "Hatari options.c";
 
 #include <ctype.h>
 #include <stdio.h>
@@ -44,6 +44,7 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "xbios.h"
 #include "stMemory.h"
 #include "tos.h"
+#include "lilo.h"
 
 
 bool bLoadAutoSave;        /* Load autosave memory snapshot at startup */
@@ -80,12 +81,12 @@ enum {
 	OPT_DRIVE_LED,
 	OPT_MAXWIDTH,
 	OPT_MAXHEIGHT,
+	OPT_ZOOM,
 	OPT_FORCEBPP,
 	OPT_DISABLE_VIDEO,
 	OPT_BORDERS,		/* ST/STE display options */
 	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
-	OPT_ZOOM,
 	OPT_VIDEO_TIMING,
 	OPT_RESOLUTION,		/* TT/Falcon display options */
 	OPT_FORCE_MAX,
@@ -100,6 +101,7 @@ enum {
 	OPT_AVI_PNG_LEVEL,
 	OPT_AVIRECORD_FPS,
 	OPT_AVIRECORD_FILE,
+	OPT_SCRSHOT_DIR,
 	OPT_JOYSTICK,		/* device options */
 	OPT_JOYSTICK0,
 	OPT_JOYSTICK1,
@@ -172,6 +174,9 @@ enum {
 #endif
 	OPT_DEBUG,
 	OPT_EXCEPTIONS,
+#if ENABLE_WINUAE_CPU
+	OPT_LILO,
+#endif
 	OPT_BIOSINTERCEPT,
 	OPT_CONOUT,
 	OPT_DISASM,
@@ -244,10 +249,21 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Show statusbar (floppy leds etc)" },
 	{ OPT_DRIVE_LED,   NULL, "--drive-led",
 	  "<bool>", "Show overlay drive led when statusbar isn't shown" },
+#if WITH_SDL2
+	{ OPT_MAXWIDTH, NULL, "--max-width",
+	  "<x>", "Maximum Hatari screen width before scaling" },
+	{ OPT_MAXHEIGHT, NULL, "--max-height",
+	  "<x>", "Maximum Hatari screen height before scaling" },
+	{ OPT_ZOOM, "-z", "--zoom",
+	  "<x>", "Hatari screen/window scaling factor (1.0 - 8.0)" },
+#else
 	{ OPT_MAXWIDTH, NULL, "--max-width",
 	  "<x>", "Maximum window width for borders & zooming" },
 	{ OPT_MAXHEIGHT, NULL, "--max-height",
 	  "<x>", "Maximum window height for borders & zooming" },
+	{ OPT_ZOOM, "-z", "--zoom",
+	  "<x>", "Double small ST/STe resolutions (1=no, 2=yes)" },
+#endif
 	{ OPT_FORCEBPP, NULL, "--bpp",
 	  "<x>", "Force internal bitdepth (x = 15/16/32, 0=disable)" },
 	{ OPT_DISABLE_VIDEO,   NULL, "--disable-video",
@@ -260,8 +276,6 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Keep desktop resolution on fullscreen" },
 	{ OPT_SPEC512, NULL, "--spec512",
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
-	{ OPT_ZOOM, "-z", "--zoom",
-	  "<x>", "Double small resolutions (1=no, 2=yes)" },
 	{ OPT_VIDEO_TIMING,   NULL, "--video-timing",
 	  "<x>", "Wakeup State for MMU/GLUE (x=ws1/ws2/ws3/ws4/random, default ws3)" },
 
@@ -296,6 +310,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Force AVI frame rate (x = 50/60/71/...)" },
 	{ OPT_AVIRECORD_FILE, NULL, "--avi-file",
 	  "<file>", "Use <file> to record AVI" },
+	{ OPT_SCRSHOT_DIR, NULL, "--screenshot-dir",
+	  "<dir>", "Save screenshots in the directory <dir>" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Devices" },
 	{ OPT_JOYSTICK,  "-j", "--joystick",
@@ -386,7 +402,7 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB ; else size in KiB)" },
 #if ENABLE_WINUAE_CPU
 	{ OPT_TT_RAM,   NULL, "--ttram",
-	  "<x>", "TT RAM size (x = size in MiB from 0 to 256)" },
+	  "<x>", "TT RAM size (x = size in MiB from 0 to 512)" },
 #endif
 	{ OPT_MEMSTATE,   NULL, "--memstate",
 	  "<file>", "Load memory snap-shot <file>" },
@@ -443,7 +459,7 @@ static const opt_t HatariOptions[] = {
 	{ OPT_SOUND,   NULL, "--sound",
 	  "<x>", "Sound frequency (x=off/6000-50066, off=fastest)" },
 	{ OPT_SOUNDBUFFERSIZE,   NULL, "--sound-buffer-size",
-	  "<x>", "Sound buffer size in ms (x=0/10-100, 0=default)" },
+	  "<x>", "Sound buffer size in ms (x=0/10-100, 0=SDL default)" },
 	{ OPT_SOUNDSYNC,   NULL, "--sound-sync",
 	  "<bool>", "Sound synchronized emulation (on|off, off=default)" },
 	{ OPT_YM_MIXING,   NULL, "--ym-mixing",
@@ -458,6 +474,9 @@ static const opt_t HatariOptions[] = {
 	  NULL, "Toggle whether CPU exceptions invoke debugger" },
 	{ OPT_EXCEPTIONS, NULL, "--debug-except",
 	  "<flags>", "Exceptions invoking debugger, see '--debug-except help'" },
+#if ENABLE_WINUAE_CPU
+	{ OPT_LILO, NULL, "--lilo", "<x>", "Boot Linux (see manual page)" },
+#endif
 	{ OPT_BIOSINTERCEPT, NULL, "--bios-intercept",
 	  "<bool>", "Enable/disable XBIOS command parsing support" },
 	{ OPT_CONOUT,   NULL, "--conout",
@@ -680,7 +699,7 @@ bool Opt_ShowError(unsigned int optid, const char *value, const char *error)
 
 /**
  * Return given value after constraining it within "min" and "max" values
- * and making it evenly divisable by "align"
+ * and making it evenly divisible by "align"
  */
 int Opt_ValueAlignMinMax(int value, int align, int min, int max)
 {
@@ -749,7 +768,7 @@ static bool Opt_Bool(const char *arg, int optid, bool *conf)
 
 
 /**
- * checks str argument agaist options of type "--option<digit>".
+ * checks str argument against options of type "--option<digit>".
  * If match is found, returns ID for that, otherwise OPT_CONTINUE
  * and OPT_ERROR for errors.
  */
@@ -1034,9 +1053,14 @@ static bool Opt_HandleArgument(const char *path)
  */
 bool Opt_ParseParameters(int argc, const char * const argv[])
 {
-	int ncpu, skips, zoom, planes, cpuclock, threshold, memsize, port, freq, temp, drive;
+	int ncpu, skips, planes, cpuclock, threshold, memsize, port, freq, temp, drive;
 	const char *errstr, *str;
 	int i, ok = true;
+#if WITH_SDL2
+	float zoom;
+#else
+	int zoom;
+#endif
 	int val;
 
 	/* Defaults for loading initial memory snap-shots */
@@ -1235,18 +1259,34 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 		case OPT_ZOOM:
+#if WITH_SDL2
+			zoom = atof(argv[++i]);
+			if (zoom < 1.0 || zoom > 8.0)
+#else
 			zoom = atoi(argv[++i]);
-			if (zoom < 1)
+			if (zoom < 1 || zoom > 2)
+#endif
 			{
 				return Opt_ShowError(OPT_ZOOM, argv[i], "Invalid zoom value");
 			}
 			ConfigureParams.Screen.nMaxWidth = NUM_VISIBLE_LINE_PIXELS;
 			ConfigureParams.Screen.nMaxHeight = NUM_VISIBLE_LINES;
+#if WITH_SDL2
+			/* double ST-low always so that resulting screen size
+			 * is approximately same size with same zoom factor
+			 * regardless of the machine or monitor type
+			 */
+			ConfigureParams.Screen.nMaxWidth *= 2;
+			ConfigureParams.Screen.nMaxHeight *= 2;
+			ConfigureParams.Screen.nZoomFactor = zoom;
+#else
+			/* zoom factor only for ST-low mode */
 			if (zoom > 1)
 			{
 				ConfigureParams.Screen.nMaxWidth *= 2;
 				ConfigureParams.Screen.nMaxHeight *= 2;
 			}
+#endif
 			ConfigureParams.Screen.nMaxHeight += STATUSBAR_MAX_HEIGHT;
 			break;
 
@@ -1336,6 +1376,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 					argv[i], sizeof(ConfigureParams.Video.AviRecordFile), NULL);
 			break;
 
+		case OPT_SCRSHOT_DIR:
+			i += 1;
+			Paths_SetScreenShotDir(argv[i]);
+			break;
+
 			/* VDI options */
 		case OPT_VDI:
 			ok = Opt_Bool(argv[++i], OPT_VDI, &ConfigureParams.Screen.bUseExtVdiResolutions);
@@ -1621,7 +1666,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_ACSIHDIMAGE:
 			i += 1;
 			str = argv[i];
-			if (strlen(str) > 2 && isdigit(str[0]) && str[1] == '=')
+			if (strlen(str) > 2 && isdigit((unsigned char)str[0]) && str[1] == '=')
 			{
 				drive = str[0] - '0';
 				if (drive < 0 || drive >= MAX_ACSI_DEVS)
@@ -1644,7 +1689,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_SCSIHDIMAGE:
 			i += 1;
 			str = argv[i];
-			if (strlen(str) > 2 && isdigit(str[0]) && str[1] == '=')
+			if (strlen(str) > 2 && isdigit((unsigned char)str[0]) && str[1] == '=')
 			{
 				drive = str[0] - '0';
 				if (drive < 0 || drive >= MAX_SCSI_DEVS)
@@ -1689,7 +1734,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_IDEBYTESWAP:
 			i += 1;
 			str = argv[i];
-			if (strlen(str) > 2 && isdigit(str[0]) && str[1] == '=')
+			if (strlen(str) > 2 && isdigit((unsigned char)str[0]) && str[1] == '=')
 			{
 				drive = str[0] - '0';
 				if (drive < 0 || drive > 1)
@@ -1878,7 +1923,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				ConfigureParams.System.nMachineType = MACHINE_MEGA_STE;
 				ConfigureParams.System.nCpuLevel = 0;
-				Configuration_ChangeCpuFreq ( 16 );
+				Configuration_ChangeCpuFreq ( 8 );
 			}
 			else if (strcasecmp(argv[i], "tt") == 0)
 			{
@@ -2073,7 +2118,26 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				        oldmask, ExceptionDebugMask);
 			}
 			break;
-
+#if ENABLE_WINUAE_CPU
+		case OPT_LILO: {
+			size_t len;
+			len = strlen(argv[++i]);
+			len += strlen(ConfigureParams.Lilo.szCommandLine);
+			if (argv[i][0] && len+2 < sizeof(ConfigureParams.Lilo.szCommandLine))
+			{
+				strcat(ConfigureParams.Lilo.szCommandLine, " ");
+				strcat(ConfigureParams.Lilo.szCommandLine, argv[i]);
+			}
+			else if (argv[i][0])
+			{
+				return Opt_ShowError(OPT_LILO, argv[i], "kernel command line too long");
+			}
+			bLoadAutoSave = false;
+			bUseLilo = true;
+			bUseTos = false;
+			break;
+		}
+#endif
 		case OPT_BIOSINTERCEPT:
 			ok = Opt_Bool(argv[++i], OPT_BIOSINTERCEPT, &bBiosIntercept);
 			Log_Printf(LOG_DEBUG, "XBIOS 11/20/255 Hatari versions %sabled: "
@@ -2163,7 +2227,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_PARSE:
 			i += 1;
-			ok = DebugUI_SetParseFile(argv[i]);
+			ok = DebugUI_AddParseFile(argv[i]);
 			break;
 
 		case OPT_SAVECONFIG:
