@@ -8,19 +8,12 @@
 
   This is similar to the printing functions, we open a direct file
   (e.g. /dev/ttyS0) and send bytes over it.
-  Using such method mimicks the ST exactly, and even allows us to connect
+  Using such method mimics the ST exactly, and even allows us to connect
   to an actual ST! To wait for incoming data, we create a thread which copies
   the bytes into an input buffer. This method fits in with the internet code
   which also reads data into a buffer.
 */
-const char RS232_fileid[] = "Hatari rs232.c : " __DATE__ " " __TIME__;
-
-#include <config.h>
-
-#if HAVE_TERMIOS_H
-# include <termios.h>
-# include <unistd.h>
-#endif
+const char RS232_fileid[] = "Hatari rs232.c";
 
 #include <SDL.h>
 #include <SDL_thread.h>
@@ -33,6 +26,15 @@ const char RS232_fileid[] = "Hatari rs232.c : " __DATE__ " " __TIME__;
 #include "mfp.h"
 #include "rs232.h"
 
+/* AmigaOS has termios.h, but no tcsetattr() and friends - d'oh! */
+#if !defined(HAVE_TCSETATTR)
+#undef HAVE_TERMIOS_H
+#endif
+
+#if HAVE_TERMIOS_H
+# include <termios.h>
+# include <unistd.h>
+#endif
 
 #define RS232_DEBUG 0
 
@@ -66,39 +68,6 @@ static inline void cfmakeraw(struct termios *termios_p)
 #endif
 
 
-#if defined(__AMIGAOS4__)
-
-// dummy functions. REMOVE THEM LATER
-
-int tcgetattr(int file_descriptor,struct termios *tios_p)
-{
-	return -1;
-}
-
-int tcsetattr(int file_descriptor,int action,struct termios *tios_p)
-{
-	return -1;
-}
-
-int cfsetospeed(struct termios *tios, speed_t ospeed)
-{
-
-	tios->c_ospeed = ospeed;
-
-	return 0;
-}
-
-
-int cfsetispeed(struct termios *tios,speed_t ispeed)
-{
-	tios->c_ispeed = ispeed;
-
-	return 0;
-}
-
-#endif  /* __AMIGAOS4__ */
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Set serial line parameters to "raw" mode.
@@ -258,17 +227,22 @@ static bool RS232_OpenCOMPort(void)
  */
 static void RS232_CloseCOMPort(void)
 {
-	if (hComIn)
-	{
-		/* Close */
-		fclose(hComIn);
-		hComIn = NULL;
-	}
+	/* Write side needs to be closed first.  Otherwise Hatari
+	 * instances at both ends of a "RS-232" FIFO file would freeze
+	 * when Hatari exists or RS-232 configuration is changed
+	 * (with this, only one of them freezes until other
+	 * end of a FIFO also closes the "device" file(s)).
+	 */
 	if (hComOut)
 	{
 		fclose(hComOut);
 		hComOut = NULL;
 	}
+	if (hComIn)
+	{
+		fclose(hComIn);
+		hComIn = NULL;
+	}
 	Dprintf(("Closed RS232 files.\n"));
 }
 
@@ -318,7 +292,7 @@ static int RS232_ThreadFunc(void *pData)
 				cInChar = iInChar;
 				RS232_AddBytesToInputBuffer(&cInChar, 1);
 				/* FIXME: Use semaphores to lock MFP variables? */
-				MFP_InputOnChannel ( MFP_INT_RCV_BUF_FULL , 0 );
+				MFP_InputOnChannel ( pMFP_Main , MFP_INT_RCV_BUF_FULL , 0 );
 				Dprintf(("RS232: Read character $%x\n", iInChar));
 				/* Sleep for a while */
 				SDL_Delay(2);
@@ -364,7 +338,7 @@ void RS232_Init(void)
 	{
 		/* Create semaphore */
 		if (pSemFreeBuf == NULL)
-			pSemFreeBuf = SDL_CreateSemaphore(MAX_RS232INPUT_BUFFER);
+			pSemFreeBuf = SDL_CreateSemaphore(MAX_RS232INPUT_BUFFER - 1);
 		if (pSemFreeBuf == NULL)
 		{
 			RS232_CloseCOMPort();
@@ -521,7 +495,7 @@ static bool RS232_SetBaudRate(int nBaud)
 		return false;
 	}
 
-	/* Set ouput speed: */
+	/* Set output speed: */
 	if (hComOut != NULL)
 	{
 		memset (&termmode, 0, sizeof(termmode));    /* Init with zeroes */
@@ -626,7 +600,7 @@ static bool RS232_TransferBytesTo(Uint8 *pBytes, int nBytes)
 		if (fwrite(pBytes, 1, nBytes, hComOut))
 		{
 			Dprintf(("RS232: Sent %i bytes ($%x ...)\n", nBytes, *pBytes));
-			MFP_InputOnChannel ( MFP_INT_TRN_BUF_EMPTY , 0 );
+			MFP_InputOnChannel ( pMFP_Main , MFP_INT_TRN_BUF_EMPTY , 0 );
 
 			return true;   /* OK */
 		}
@@ -682,7 +656,7 @@ static bool RS232_GetStatus(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Read from the Syncronous Character Register.
+ * Read from the Synchronous Character Register.
  */
 void RS232_SCR_ReadByte(void)
 {
@@ -693,7 +667,7 @@ void RS232_SCR_ReadByte(void)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Write to the Syncronous Character Register.
+ * Write to the Synchronous Character Register.
  */
 void RS232_SCR_WriteByte(void)
 {
@@ -761,7 +735,7 @@ void RS232_RSR_WriteByte(void)
  * Read from the Transmitter Status Register.
  * When RS232 emulation is not enabled, we still return 0x80 to allow
  * some games to work when they don't require send/receive on the RS232 port
- * (eg : 'Treasure Trap', 'The Deep' write some debug informations to RS232)
+ * (eg : 'Treasure Trap', 'The Deep' write some debug information to RS232)
  */
 void RS232_TSR_ReadByte(void)
 {
@@ -801,7 +775,7 @@ void RS232_UDR_ReadByte(void)
 	if (RS232_GetStatus())              /* More data waiting? */
 	{
 		/* Yes, generate another interrupt. */
-		MFP_InputOnChannel ( MFP_INT_RCV_BUF_FULL , 0 );
+		MFP_InputOnChannel ( pMFP_Main , MFP_INT_RCV_BUF_FULL , 0 );
 	}
 }
 
