@@ -6,7 +6,7 @@
 
   ST Memory access functions.
 */
-const char STMemory_fileid[] = "Hatari stMemory.c : " __DATE__ " " __TIME__;
+const char STMemory_fileid[] = "Hatari stMemory.c";
 
 #include "stMemory.h"
 #include "configuration.h"
@@ -54,6 +54,12 @@ static Uint32	STMemory_MMU_Translate_Addr_STF ( Uint32 addr_logical , int RAM_Ba
 static Uint32	STMemory_MMU_Translate_Addr_STE ( Uint32 addr_logical , int RAM_Bank_Size , int MMU_Bank_Size );
 
 
+#define	DMA_READ_WORD_BUS_ERR	0x0000		/* This value is returned when reading a word using DMA (blitter, sound) */
+						/* in a region that would cause a bus error */
+						/* [NP] FIXME : for now we return a constant, but it should depend on the bus activity */
+#define	DMA_READ_BYTE_BUS_ERR	0x00
+
+
 
 /**
  * Set default value for MMU bank size and RAM bank size
@@ -116,7 +122,15 @@ bool STMemory_SafeClear(Uint32 addr, unsigned int len)
 
 	if (STMemory_CheckAreaType(addr, len, ABFLAG_RAM))
 	{
-		memset(&STRam[addr], 0, len);
+		if (addr + len < 0x1000000)
+		{
+			memset(&STRam[addr], 0, len);
+		}
+		else
+		{
+			assert(TTmemory && addr + len <= TTmem_size + 0x1000000);
+			memset(&TTmemory[addr - 0x1000000], 0, len);
+		}
 		return true;
 	}
 	Log_Printf(LOG_WARN, "Invalid RAM clear range 0x%x+%i!\n", addr, len);
@@ -124,7 +138,7 @@ bool STMemory_SafeClear(Uint32 addr, unsigned int len)
 	for (end = addr + len; addr < end; addr++)
 	{
 		if (STMemory_CheckAreaType(addr, 1, ABFLAG_RAM))
-			STRam[addr] = 0;
+			put_byte(addr, 0);
 	}
 	return false;
 }
@@ -147,15 +161,23 @@ bool STMemory_SafeCopy(Uint32 addr, Uint8 *src, unsigned int len, const char *na
 
 	if ( STMemory_CheckAreaType ( addr, len, ABFLAG_RAM ) )
 	{
-		memcpy(&STRam[addr], src, len);
+		if (addr + len < 0x1000000)
+		{
+			memcpy(&STRam[addr], src, len);
+		}
+		else
+		{
+			assert(TTmemory && addr + len <= TTmem_size + 0x1000000);
+			memcpy(&TTmemory[addr - 0x1000000], src, len);
+		}
 		return true;
 	}
 	Log_Printf(LOG_WARN, "Invalid '%s' RAM range 0x%x+%i!\n", name, addr, len);
 
-	for (end = addr + len; addr < end; addr++, src++)
+	for (end = addr + len; addr < end; addr++)
 	{
 		if ( STMemory_CheckAreaType ( addr, 1, ABFLAG_RAM ) )
-			STRam[addr] = *src;
+			put_byte(addr, *src++);
 	}
 	return false;
 }
@@ -429,6 +451,49 @@ void STMemory_SetDefaultConfig(void)
 	STMemory_WriteLong(0x4c2, ConnectedDriveMask);
 }
 
+/**
+ * Called after machine type is fixed, to correct ST-RAM amount to machine
+ * specific value, when machine doesn't support all values accepted by Hatari.
+ *
+ * Returns resulting STRamEnd value.
+ */
+int STMemory_CorrectSTRamSize(void)
+{
+	int STRamSize_KB = ConfigureParams.Memory.STRamSize_KB;
+
+	if (Config_IsMachineFalcon())
+	{
+		/* These values need to match ones used in STMemory_SetDefaultConfig()
+		 * above.
+		 *
+		 * Check ranges which might have non-supported memory values:
+		 *   10MB, 6MB & 2.5MB
+		 */
+		if (STRamSize_KB > 8*1024 && STRamSize_KB < 14*1024)
+			STRamSize_KB = 14*1024;
+		else if (STRamSize_KB > 4*1024 && STRamSize_KB < 8*1024)
+			STRamSize_KB = 8*1024;
+		else if (STRamSize_KB > 2*1024 && STRamSize_KB < 4*1024)
+			STRamSize_KB = 4*1024;
+
+		if (STRamSize_KB != ConfigureParams.Memory.STRamSize_KB)
+		{
+			Log_Printf(LOG_WARN, "unsupported Falcon ST-RAM amount %d, changing to %d KB\n",
+				   ConfigureParams.Memory.STRamSize_KB, STRamSize_KB);
+			ConfigureParams.Memory.STRamSize_KB = STRamSize_KB;
+		}
+	}
+	else if (Config_IsMachineMegaSTE() || Config_IsMachineTT())
+	{
+		if (STRamSize_KB > 10*1024)
+		{
+			Log_Printf(LOG_INFO, "maximum MegaSTE/TT ST-RAM amount is 10MB, not %dMB\n",
+				   STRamSize_KB/1024);
+		}
+	}
+	return STRamSize_KB * 1024;
+}
+
 
 /**
  * Check that the region of 'size' starting at 'addr' is entirely inside
@@ -452,14 +517,27 @@ bool	STMemory_CheckAreaType ( Uint32 addr , int size , int mem_type )
 
 
 /**
- * Check if an address points to a memory region that causes bus error
+ * Check if an address access would cause a bus error (read or write)
  * This is used for blitter and other DMA chips that should not cause
- * a bus error when accessing such regions (on the contrary of the CPU)
- * Returns true if region gives bus error
+ * a bus error when accessing directly such regions (on the contrary of the CPU)
+ * Bus error can come from :
+ *  - an access to a bus error region
+ *  - an access to a part of the IO region that cause a bus error
+ * Returns true if address would give a bus error
  */
-bool	STMemory_CheckRegionBusError ( Uint32 addr )
+bool	STMemory_CheckAddrBusError ( Uint32 addr )
 {
-	return memory_region_bus_error ( addr );
+	/* Check if it's a whole "bus error" region */
+	if ( memory_region_bus_error ( addr ) )
+		return true;
+
+	/* In case of IO region, bus error can happen at various addresses, depending on the machine type */
+	if ( memory_region_iomem ( addr ) && IoMem_CheckBusError ( addr ) )
+		return true;
+
+	/* TODO : in case of the Falcon, we should also check IDE region 0xF0xxxx for possible bus error */
+
+	return false;
 }
 
 
@@ -484,6 +562,29 @@ void	*STMemory_STAddrToPointer ( Uint32 addr )
 }
 
 
+/**
+ * Get the host memory pointer of a NUL-terminated string in the ST memory,
+ * or NULL if the whole string memory is not accessible or if the string
+ * is too big (i.e. rather garbage than a real string).
+ */
+char *STMemory_GetStringPointer(uint32_t addr)
+{
+	int idx = 0;
+
+	do
+	{
+		if (!STMemory_CheckAreaType(addr + idx, 1,  ABFLAG_RAM | ABFLAG_ROM))
+		{
+			return NULL;
+		}
+		if (*(char *)STMemory_STAddrToPointer(addr + idx) == '\0')
+		{
+			return STMemory_STAddrToPointer(addr);
+		}
+	} while (idx++ < 0x10000);
+
+	return NULL;
+}
 
 /**
  * Those functions are directly accessing the memory of the corresponding
@@ -586,6 +687,59 @@ Uint8	STMemory_ReadByte ( Uint32 addr )
 
 
 
+/**
+ * Access memory when using DMA
+ * Contrary to the CPU, when DMA is used there should be no bus error
+ */
+Uint16	STMemory_DMA_ReadWord ( Uint32 addr )
+{
+	Uint16 value;
+
+	/* When reading from a bus error region, just return a constant */
+	if ( STMemory_CheckAddrBusError ( addr ) )
+		value = DMA_READ_WORD_BUS_ERR;
+	else
+		value = (Uint16)get_word ( addr );
+//fprintf ( stderr , "readw %x %x %x\n" , addr , value , STMemory_CheckAddrBusError(addr) );
+	return value;
+}
+
+
+void	STMemory_DMA_WriteWord ( Uint32 addr , Uint16 value )
+{
+	/* Call put_word only if the address doesn't point to a bus error region */
+	/* (also see SysMem_wput for addr < 0x8) */
+	if ( STMemory_CheckAddrBusError ( addr ) == false )
+		put_word ( addr , (Uint32)(value) );
+//fprintf ( stderr , "writew %x %x %x\n" , addr , value , STMemory_CheckAddrBusError(addr) );
+}
+
+
+Uint8	STMemory_DMA_ReadByte ( Uint32 addr )
+{
+	Uint8 value;
+
+	/* When reading from a bus error region, just return a constant */
+	if ( STMemory_CheckAddrBusError ( addr ) )
+		value = DMA_READ_BYTE_BUS_ERR;
+	else
+		value = (Uint8)get_byte ( addr );
+//fprintf ( stderr , "readb %x %x %x\n" , addr , value , STMemory_CheckAddrBusError(addr) );
+	return value;
+}
+
+
+void	STMemory_DMA_WriteByte ( Uint32 addr , Uint8 value )
+{
+	/* Call put_word only if the address doesn't point to a bus error region */
+	/* (also see SysMem_wput for addr < 0x8) */
+	if ( STMemory_CheckAddrBusError ( addr ) == false )
+		put_byte ( addr , (Uint32)(value) );
+//fprintf ( stderr , "writeb %x %x %x\n" , addr , value , STMemory_CheckAddrBusError(addr) );
+}
+
+
+
 
 /*
 
@@ -771,13 +925,13 @@ int	STMemory_RAM_Validate_Size_KB ( int TotalMem )
 	/* Old format where ST RAM size was in MB between 0 and 14 */
 	if ( TotalMem == 0 )
 		return 512;
-	else if ( TotalMem <= 14 )
-		return TotalMem * 1024;
+	if ( TotalMem <= 14 )
+		TotalMem *= 1024;
 
 	/* New format where ST RAM size is in KB */
-	else if (  ( TotalMem ==  128 ) || ( TotalMem ==  256 ) || ( TotalMem ==  512 ) || ( TotalMem ==  640 )
+	if (  ( TotalMem ==  128 ) || ( TotalMem ==  256 ) || ( TotalMem ==  512 ) || ( TotalMem ==  640 )
 		|| ( TotalMem == 1024 ) || ( TotalMem == 2048 ) || ( TotalMem == 2176 ) || ( TotalMem == 2560 )
-		|| ( TotalMem == 4096 )	|| ( TotalMem == 8*1024 ) || ( TotalMem == 14*1024 ) )
+		|| ( TotalMem == 4096 ) || ( TotalMem == 8*1024 ) || ( TotalMem == 10*1024 ) || ( TotalMem == 14*1024 ) )
 		return TotalMem;
 
 	return -1;
