@@ -11,7 +11,7 @@
 
    Read unzip.h for more info
 */
-const char Unzip_fileid[] = "Hatari unzip.c : " __DATE__ " " __TIME__;
+const char Unzip_fileid[] = "Hatari unzip.c";
 
 #include <config.h>
 
@@ -20,6 +20,7 @@ const char Unzip_fileid[] = "Hatari unzip.c : " __DATE__ " " __TIME__;
 #include <stddef.h>
 #include <string.h>
 #include <zlib.h>
+#include <errno.h>
 
 #if HAVE_STRINGS_H
 #include <strings.h>
@@ -27,13 +28,6 @@ const char Unzip_fileid[] = "Hatari unzip.c : " __DATE__ " " __TIME__;
 
 #include "unzip.h"
 
-#ifdef NO_ERRNO_H
-    extern int errno;
-#else
-#   include <errno.h>
-#endif
-
-
 #ifndef local
 #  define local static
 #endif
@@ -218,7 +212,7 @@ local int unzlocal_getLong (FILE* fin, uLong *pX)
  *  If iCaseSenisivity = 1, comparison is case sensitivity (like strcmp)
  *  If iCaseSenisivity = 2, comparison is not case sensitivity (like strcmpi
  *                                                               or strcasecmp)
- *  If iCaseSenisivity = 0, case sensitivity is defaut of your operating system
+ *  If iCaseSenisivity = 0, case sensitivity is default of your operating system
  *       (like 1 on Unix, 2 on Windows)
  *
  */
@@ -312,9 +306,9 @@ unzFile ZEXPORT unzOpen (const char *path)
 	FILE * fin ;
 
 	uLong number_disk;          /* number of the current dist, used for 
-								   spaning ZIP, unsupported, always 0*/
+								   spanning ZIP, unsupported, always 0*/
 	uLong number_disk_with_CD;  /* number the the disk with central dir, used
-								   for spaning ZIP, unsupported, always 0*/
+								   for spanning ZIP, unsupported, always 0*/
 	uLong number_entry_CD;      /* total number of entries in
 	                               the central dir 
 	                               (same than number_entry on nospan) */
@@ -920,7 +914,7 @@ int ZEXPORT unzOpenCurrentFile (unzFile file)
  * buf contain buffer where data must be copied
  * len the size of buf.
  *
- * return the number of byte copied if somes bytes are copied
+ * return the number of byte copied if some bytes are copied
  * return 0 if the end of file was reached
  * return <0 with error code if there is an error
  *   (UNZ_ERRNO for IO error, or zLib error for uncompress error)
